@@ -72,48 +72,9 @@
 #define N_ELEMENTS_BETWEEN_PROGRESS_UPDATES   20
 
 static excel_iconv_t current_workbook_iconv = NULL;
-/**
- *  This function converts simple strings...
- **/
-int
-biff_convert_text (char **buf, const char *txt, MsBiffVersion ver)
-{
-	guint32 lp, len;
 
-	g_return_val_if_fail (txt, 0);
-
-	len = strlen (txt);
-	if (len == 0)
-		*buf = g_strdup ("");
-	else if (ver >= MS_BIFF_V8) {	/* unicode */
-		wchar_t* wcbuf;
-		guint16 *outbuf;
-		len = mbstowcs(NULL, txt, 0);
-		g_return_val_if_fail (len > 0, 0);
-		wcbuf = g_new(wchar_t, len + 1);
-		mbstowcs(wcbuf,txt,len + 1);
-
-		outbuf = g_new(guint16, len);
-		*buf = (char *)outbuf;
-		for (lp = 0; lp < len; lp++) {
-			outbuf[lp] = wcbuf[lp];
-		}		
-		g_free(wcbuf);
-		len = lp * 2;
-	} else {
-		size_t inbufleft = len, outbufleft = len*8;
-		char *outbufptr;
-		char const * inbufptr = txt;
-
-		*buf = g_new(char, outbufleft);
-		outbufptr = *buf;
-
-		excel_iconv (current_workbook_iconv, &inbufptr, &inbufleft, 
-			     &outbufptr, &outbufleft);
-		len = outbufptr - *buf;
-	};
-	return len;
-}
+static guint style_color_to_rgb888 (const StyleColor *c);
+static gint  palette_get_index (ExcelWorkbook *wb, guint c);
 
 /**
  *  This function writes simple strings...
@@ -121,27 +82,23 @@ biff_convert_text (char **buf, const char *txt, MsBiffVersion ver)
  *  it returns the length of the string.
  **/
 int
-biff_put_text (BiffPut *bp, const char *txt, int len, MsBiffVersion ver,
+biff_put_text (BiffPut *bp, const char *txt, MsBiffVersion ver,
 	       gboolean write_len, PutType how)
 {
 #define BLK_LEN 16
 
 	guint8 data[BLK_LEN];
-	guint32 ans;
-	int lp;
+	guint32 lp, len, ans;
 
 	gboolean sixteen_bit_len;
 	gboolean unicode;
 	guint32  off;
 
 	g_return_val_if_fail (bp, 0);
-
-	if (txt == NULL) {
-		g_warning ("writing NULL string as \"\"");
-		txt = "";
-	}
+	g_return_val_if_fail (txt, 0);
 
 	ans = 0;
+	len = strlen (txt);
 /*	printf ("Write '%s' len = %d\n", txt, len); */
 
 	if ((how == AS_PER_VER &&
@@ -157,46 +114,49 @@ biff_put_text (BiffPut *bp, const char *txt, int len, MsBiffVersion ver,
 		unicode = FALSE;
 
 	off = 0;
-	if (unicode) {
-		guint16 *buf = (guint16 *)txt;
-
-		if (write_len) {
-			if (sixteen_bit_len) {
-				MS_OLE_SET_GUINT16 (data, len/2);
-				off = 2;
-			} else {
-				g_return_val_if_fail (len/2<256, 0);
-				MS_OLE_SET_GUINT8  (data, len/2);
-				off = 1;
-			}
+	if (write_len) {
+		if (sixteen_bit_len) {
+			MS_OLE_SET_GUINT16 (data, len);
+			off = 2;
+		} else {
+			g_return_val_if_fail (len<256, 0);
+			MS_OLE_SET_GUINT8  (data, len);
+			off = 1;
 		}
+	}
+
+	if (unicode) {
 		MS_OLE_SET_GUINT8  (data + off, 0x0);
 		off++;
-		ms_biff_put_var_write (bp, data, off);
+	}
+	ms_biff_put_var_write (bp, data, off);
 
-		for (lp = 0; lp < len/2; lp++) {
-			MS_OLE_SET_GUINT16 (data, buf[lp]);
+	if (unicode) {
+		wchar_t* wcbuf = g_new(wchar_t,len);
+		len = mbstowcs(wcbuf,txt,len);
+		for (lp = 0; lp < len; lp++) {
+			MS_OLE_SET_GUINT16 (data, wcbuf[lp]);
 			ms_biff_put_var_write (bp, data, 2);
 		}
+		g_free(wcbuf);
+		lp *= 2;
 	} else {
-		if (write_len) {
-			if (sixteen_bit_len) {
-				MS_OLE_SET_GUINT16 (data, len);
-				off = 2;
-			} else {
-				g_return_val_if_fail (len<256, 0);
-				MS_OLE_SET_GUINT8  (data, len);
-				off = 1;
-			}
-			ms_biff_put_var_write (bp, data, off);
-		}
+		size_t inbufleft = len, outbufleft = len*8;
+		char* mbbuf = g_new(char, outbufleft);
+		char *outbufptr = mbbuf;
+		char const * inbufptr = txt;
+		guint32 retlen;
 
-		for (lp = 0; lp < len; lp++) {
-			MS_OLE_SET_GUINT8 (data, txt[lp]);
-			ms_biff_put_var_write (bp, data, 1);			
+		excel_iconv (current_workbook_iconv, &inbufptr, &inbufleft,
+			     &outbufptr, &outbufleft);
+		retlen = outbufptr - mbbuf;
+		for (lp = 0; lp < retlen; lp++) {
+			MS_OLE_SET_GUINT8 (data, mbbuf[lp]);
+			ms_biff_put_var_write (bp, data, 1);
 		}
+		g_free(mbbuf);
 	};
-	return off + len;
+	return off + lp;
 	/* An attempt at efficiency */
 /*	chunks = len/BLK_LEN;
 	pos    = 0;
@@ -363,7 +323,7 @@ write_setup (BiffPut *bp, ExcelSheet *esheet)
  * XL doesn't write externsheets as often as we do. The previous version of
  * this function causes XL to crash when we exported a spreadsheet function
  * which is not an XL builtin. This version is still wrong, but won't crash
- * XL. 
+ * XL.
  */
 static void
 write_externsheets (BiffPut *bp, ExcelWorkbook *wb, ExcelSheet *ignore)
@@ -391,26 +351,24 @@ write_externsheets (BiffPut *bp, ExcelWorkbook *wb, ExcelSheet *ignore)
 
 	if (externcount == 0)
 		return;
-	
+
 	data = ms_biff_put_len_next (bp, BIFF_EXTERNCOUNT, 2);
 	MS_OLE_SET_GUINT16(data, externcount);
 	ms_biff_put_commit (bp);
 
 	for (lp = 0; lp < num_sheets; lp++) {
 		ExcelSheet *esheet = g_ptr_array_index (wb->sheets, lp);
-		gint len;
-		char *buf;
+		gint len = strlen (esheet->gnum_sheet->name_quoted);
 		guint8 data[8];
 
 		if (esheet == ignore) continue;
 
-		len = biff_convert_text (&buf, esheet->gnum_sheet->name_quoted, wb->ver);
 		ms_biff_put_var_next (bp, BIFF_EXTERNSHEET);
 		MS_OLE_SET_GUINT8(data, len);
 		MS_OLE_SET_GUINT8(data + 1, 3); /* Magic */
 		ms_biff_put_var_write (bp, data, 2);
-		biff_put_text (bp, buf, len, wb->ver, FALSE, AS_PER_VER);
-		g_free(buf);
+		biff_put_text (bp, esheet->gnum_sheet->name_unquoted,
+			       wb->ver, FALSE, AS_PER_VER);
 		ms_biff_put_commit (bp);
 	}
 }
@@ -464,6 +422,9 @@ write_window2 (BiffPut *bp, MsBiffVersion ver, ExcelSheet *esheet)
 	guint8 *data;
 	CellPos top_left;
 	Sheet *sheet = esheet->gnum_sheet;
+	StyleColor *sheet_auto   = sheet_style_get_auto_pattern_color (sheet);
+	StyleColor *default_auto = style_color_auto_pattern ();
+	guint32 biff_pat_col = 0x40;	/* default grid color index == auto */
 
 	if (sheet->display_formulas)
 		options |= 0x0001;
@@ -478,6 +439,15 @@ write_window2 (BiffPut *bp, MsBiffVersion ver, ExcelSheet *esheet)
 		top_left = sheet->initial_top_left;	/* belongs in sheetView */
 	if (!sheet->hide_zero)
 		options |= 0x0010;
+	/* Grid / auto pattern color */
+	if (!style_color_equal (sheet_auto, default_auto)) {
+		biff_pat_col = style_color_to_rgb888 (sheet_auto);
+		if (ver > MS_BIFF_V7) {
+			biff_pat_col = palette_get_index (esheet->wb,
+							  biff_pat_col);
+		}
+		options &= ~0x0020;
+	}
 	if (sheet == wb_view_cur_sheet (esheet->wb->gnum_wb_view))
 		options |= 0x600; /* assume selected if it is current */
 
@@ -487,20 +457,22 @@ write_window2 (BiffPut *bp, MsBiffVersion ver, ExcelSheet *esheet)
 		MS_OLE_SET_GUINT16 (data +  0, options);
 		MS_OLE_SET_GUINT16 (data +  2, top_left.row);
 		MS_OLE_SET_GUINT16 (data +  4, top_left.col);
-		MS_OLE_SET_GUINT32 (data +  6, 0x40);	/* grid color index (0x40 == auto) */
+		MS_OLE_SET_GUINT32 (data +  6, biff_pat_col);
 	} else {
 		data = ms_biff_put_len_next (bp, 0x200|BIFF_WINDOW2, 18);
 
 		MS_OLE_SET_GUINT16 (data +  0, options);
 		MS_OLE_SET_GUINT16 (data +  2, top_left.row);
 		MS_OLE_SET_GUINT16 (data +  4, top_left.col);
-		MS_OLE_SET_GUINT32 (data +  6, 0x40);	/* grid color index (0x40 == auto) */
+		MS_OLE_SET_GUINT32 (data +  6, biff_pat_col);
 		MS_OLE_SET_GUINT16 (data + 10, 0x1);	/* print preview 100% */
 		MS_OLE_SET_GUINT16 (data + 12, 0x0);	/* FIXME : why 0? */
 		MS_OLE_SET_GUINT32 (data + 14, 0x0);	/* reserved 0 */
 	}
 	ms_biff_put_commit (bp);
 
+	style_color_unref (sheet_auto);
+	style_color_unref (default_auto);
 	return (options & 0x0008);
 }
 
@@ -598,15 +570,14 @@ write_names (BiffPut *bp, ExcelWorkbook *wb)
 			ms_biff_put_var_next (bp, BIFF_NAME);
 
 		text = expr_name->name->str;
-		name_len = biff_convert_text (&text, expr_name->name->str, wb->ver);
+		name_len = strlen (expr_name->name->str);
 
 		memset (data0, 0, sizeof (data0));
 		MS_OLE_SET_GUINT8 (data0 + 3, name_len); /* name_len */
 
 		/* This code will only work for MS_BIFF_V7. */
 		ms_biff_put_var_write (bp, data0, 14);
-		biff_put_text (bp, text, name_len, wb->ver, FALSE, AS_PER_VER);
-		g_free(text);
+		biff_put_text (bp, text, wb->ver, FALSE, AS_PER_VER);
 		ms_biff_put_var_seekto (bp, 14 + name_len);
 		len = ms_excel_write_formula (bp, esheet,
 			expr_name->t.expr_tree, 0, 0, 0);
@@ -627,18 +598,14 @@ write_bits (BiffPut *bp, ExcelWorkbook *wb, MsBiffVersion ver)
 {
 	guint8 *data;
 	char const *team = "The Gnumeric Development Team";
-	gint len;
-	char *buf;
 	guint8 pad [WRITEACCESS_LEN];
 
 	/* See: S59E1A.HTM */
-	len = biff_convert_text (&buf, team, ver);
-	g_assert (len < WRITEACCESS_LEN);
+	g_assert (strlen (team) < WRITEACCESS_LEN);
 	memset (pad, ' ', sizeof pad);
 	ms_biff_put_var_next (bp, BIFF_WRITEACCESS);
-	biff_put_text (bp, buf, len, ver, TRUE, AS_PER_VER);
-	g_free(buf);
-	ms_biff_put_var_write (bp, pad, WRITEACCESS_LEN - len - 1);
+	biff_put_text (bp, team, ver, TRUE, AS_PER_VER);
+	ms_biff_put_var_write (bp, pad, WRITEACCESS_LEN - strlen (team) - 1);
 	ms_biff_put_commit (bp);
 
 	/* See: S59D66.HTM */
@@ -737,7 +704,7 @@ ms_excel_write_get_sheet_idx (ExcelWorkbook *wb, Sheet *gnum_sheet)
 		if (esheet->gnum_sheet == gnum_sheet)
 			return lp;
 	}
-	g_warning ("No associated esheet for %p\n", (void *)gnum_sheet);
+	g_warning ("No associated esheet for %p\n", gnum_sheet);
 	return 0;
 }
 
@@ -759,8 +726,6 @@ biff_boundsheet_write_first (BiffPut *bp, MsBiffFileType type,
 			     char *name, MsBiffVersion ver)
 {
 	guint32 pos;
-	gint len;
-	char *buf;
 	guint8 data[16];
 
 	ms_biff_put_var_next (bp, BIFF_BOUNDSHEET);
@@ -786,9 +751,8 @@ biff_boundsheet_write_first (BiffPut *bp, MsBiffFileType type,
 	}
 	MS_OLE_SET_GUINT8 (data+5, 0); /* Visible */
 	ms_biff_put_var_write (bp, data, 6);
-	len = biff_convert_text(&buf, name, ver);
-	biff_put_text (bp, buf, len, ver, TRUE, AS_PER_VER);
-	g_free(buf);
+
+	biff_put_text (bp, name, ver, TRUE, AS_PER_VER);
 
 	ms_biff_put_commit (bp);
 	return pos;
@@ -827,7 +791,7 @@ palette_color_to_int (const EXCEL_PALETTE_ENTRY *c)
  * Convert StyleColor to guint representation used in BIFF file
  **/
 static guint
-style_color_to_int (const StyleColor *c)
+style_color_to_rgb888 (const StyleColor *c)
 {
 	return ((c->blue & 0xff00) << 8) + (c->green & 0xff00) + (c->red >> 8);
 
@@ -852,7 +816,7 @@ log_put_color (guint c, gboolean was_added, gint index, const char *tmpl)
 /**
  * Put Excel default colors to palette table
  *
- * Ensure that black with index 8 can't be swapped out.
+ * Ensure that black and white can't be swapped out.
  **/
 static void
 palette_put_defaults (ExcelWorkbook *wb)
@@ -868,8 +832,10 @@ palette_put_defaults (ExcelWorkbook *wb)
 				   GUINT_TO_POINTER (num), FALSE,
 				   (AfterPutFunc) log_put_color,
 				   "Default color %d - 0x%6.6x\n");
-		wb->pal->entry_in_use[i] =
-			(i == PALETTE_ALSO_BLACK) ? TRUE : FALSE;
+		if ((i == PALETTE_BLACK) || (i == PALETTE_WHITE))
+			wb->pal->entry_in_use[i] = TRUE;
+		else
+			wb->pal->entry_in_use[i] = FALSE;
 	}
 }
 
@@ -920,15 +886,15 @@ palette_get_index (ExcelWorkbook *wb, guint c)
 	TwoWayTable *twt = wb->pal->two_way_table;
 
 	if (c == 0) {
-		idx = 0;
+		idx = PALETTE_BLACK;
 	} else if (c == 0xffffff) {
-		idx = 1;
+		idx = PALETTE_WHITE;
 	} else {
 		idx = two_way_table_key_to_idx (twt, (gconstpointer) c);
 		if (idx < EXCEL_DEF_PAL_LEN) {
 			idx += 8;
 		} else {
-			idx = 0;
+			idx = PALETTE_BLACK;
 		}
 	}
 
@@ -942,7 +908,7 @@ static void
 put_color (ExcelWorkbook *wb, const StyleColor *c)
 {
 	TwoWayTable *twt = wb->pal->two_way_table;
-	gpointer pc = GUINT_TO_POINTER (style_color_to_int (c));
+	gpointer pc = GUINT_TO_POINTER (style_color_to_rgb888 (c));
 	gint idx;
 
 	two_way_table_put (twt, pc, TRUE,
@@ -950,9 +916,8 @@ put_color (ExcelWorkbook *wb, const StyleColor *c)
 			   "Found unique color %d - 0x%6.6x\n");
 
 	idx = two_way_table_key_to_idx (twt, pc);
-	if (idx >= 0 && idx < EXCEL_DEF_PAL_LEN) {
+	if (idx >= 0 && idx < EXCEL_DEF_PAL_LEN)
 		wb->pal->entry_in_use [idx] = TRUE; /* Default entry in use */
-	}
 }
 
 /**
@@ -1064,7 +1029,7 @@ excel_font_to_string (const ExcelFont *f)
 	static char buf[96];
 	guint nused;
 
-	nused = snprintf (buf, sizeof buf, "%s, %g", sf->font_name, sf->size);
+	nused = snprintf (buf, sizeof buf, "%s, %g", sf->font_name, sf->size_pts);
 
 	if (nused < sizeof buf && sf->is_bold)
 		nused += snprintf (buf + nused, sizeof buf - nused, ", %s",
@@ -1110,7 +1075,8 @@ excel_font_new (MStyle *st)
 	f = g_new (ExcelFont, 1);
 	f->style_font = mstyle_get_font (st, 1.0);
 	c = mstyle_get_color (st, MSTYLE_COLOR_FORE);
-	f->color = style_color_to_int (c);
+	f->color = style_color_to_rgb888 (c);
+	f->is_auto = c->is_auto;
 	f->underline     = mstyle_get_font_uline (st);
 	f->strikethrough = mstyle_get_font_strike (st);
 
@@ -1143,7 +1109,8 @@ excel_font_hash (gconstpointer f)
 
 	if (f)
 		res = style_font_hash_func (font->style_font) ^ font->color
-			^ font->underline ^ font->strikethrough;
+			^ font->is_auto ^ (font->underline << 1)
+			^ (font->strikethrough << 2);
 
 	return res;
 }
@@ -1169,6 +1136,7 @@ excel_font_equal (gconstpointer a, gconstpointer b)
 		const ExcelFont *fb  = (const ExcelFont *) b;
 		res = style_font_equal (fa->style_font, fb->style_font)
 			&& (fa->color == fb->color)
+			&& (fa->is_auto == fb->is_auto)
 			&& (fa->underline == fb->underline)
 			&& (fa->strikethrough == fb->strikethrough);
 	}
@@ -1237,7 +1205,7 @@ fonts_get_index (ExcelWorkbook *wb, const ExcelFont *f)
  * font is added. Free resources when it was already there.
  **/
 static void
-after_put_font (ExcelFont *f, gboolean was_added, gint index, gpointer dummy)
+after_put_font (ExcelFont *f, gboolean was_added, gint index, gconstpointer dummy)
 {
 	if (was_added) {
 		d (1, printf ("Found unique font %d - %s\n",
@@ -1294,9 +1262,9 @@ write_font (BiffPut *bp, ExcelWorkbook *wb, const ExcelFont *f)
 {
 	guint8 data[64];
 	StyleFont  *sf  = f->style_font;
-	guint32 size  = sf->size * 20;
+	guint32 size_pts  = sf->size_pts * 20;
 	guint16 grbit = 0;
-	guint16 color = palette_get_index (wb, f->color);
+	guint16 color;
 
 	guint16 boldstyle = 0x190; /* Normal boldness */
 	guint16 subsuper  = 0;   /* 0: Normal, 1; Super, 2: Sub script*/
@@ -1305,9 +1273,10 @@ write_font (BiffPut *bp, ExcelWorkbook *wb, const ExcelFont *f)
 	guint8  family    = 0;
 	guint8  charset   = 0;	 /* Seems OK. */
 	char    *font_name = sf->font_name;
-	gint    len;
-	char    *buf;
 
+	color = f->is_auto
+		? PALETTE_AUTO_FONT
+		: palette_get_index (wb, f->color);
 	d (1, printf ("Writing font %s, color idx %u\n",
 		      excel_font_to_string (f), color););
 
@@ -1319,7 +1288,7 @@ write_font (BiffPut *bp, ExcelWorkbook *wb, const ExcelFont *f)
 		boldstyle = 0x2bc;
 
 	ms_biff_put_var_next (bp, BIFF_FONT);
-	MS_OLE_SET_GUINT16 (data + 0, size);
+	MS_OLE_SET_GUINT16 (data + 0, size_pts);
 	MS_OLE_SET_GUINT16 (data + 2, grbit);
 	MS_OLE_SET_GUINT16 (data + 4, color);
 	MS_OLE_SET_GUINT16 (data + 6, boldstyle);
@@ -1329,9 +1298,8 @@ write_font (BiffPut *bp, ExcelWorkbook *wb, const ExcelFont *f)
 	MS_OLE_SET_GUINT8  (data + 12, charset);
 	MS_OLE_SET_GUINT8  (data +13, 0);
 	ms_biff_put_var_write (bp, data, 14);
-	len = biff_convert_text(&buf, font_name, wb->ver);
-	biff_put_text (bp, buf, len, wb->ver, TRUE, EIGHT_BIT);
-	g_free(buf);
+
+	biff_put_text (bp, font_name, wb->ver, TRUE, EIGHT_BIT);
 
 	ms_biff_put_commit (bp);
 }
@@ -1508,8 +1476,6 @@ gather_formats (ExcelWorkbook *wb)
 static void
 write_format (BiffPut *bp, ExcelWorkbook *wb, int fidx)
 {
-	gint len;
-	char *buf;
 	guint8 data[64];
 	StyleFormat const *sf = formats_get_format(wb, fidx);
 
@@ -1526,11 +1492,9 @@ write_format (BiffPut *bp, ExcelWorkbook *wb, int fidx)
 	MS_OLE_SET_GUINT16 (data, fidx);
 	ms_biff_put_var_write (bp, data, 2);
 
-	len = biff_convert_text(&buf, format, MS_BIFF_V7);
-	biff_put_text (bp, buf, len, MS_BIFF_V7, TRUE, AS_PER_VER);
-	g_free (buf);
-	g_free (format);
+	biff_put_text (bp, format, MS_BIFF_V7, TRUE, AS_PER_VER);
 	ms_biff_put_commit (bp);
+	g_free (format);
 }
 
 /**
@@ -1676,7 +1640,7 @@ xf_free (ExcelWorkbook *wb)
  * style is added. Add a reference when putting a style into the table.
  **/
 static void
-after_put_mstyle (MStyle *st, gboolean was_added, gint index, gpointer dummy)
+after_put_mstyle (MStyle *st, gboolean was_added, gint index, gconstpointer dummy)
 {
 	if (was_added) {
 		d (1, {
@@ -1745,11 +1709,12 @@ pre_cell (gconstpointer dummy, Cell *cell, ExcelSheet *esheet)
 
 	/* For the general format XL assigns the parse format */
 	cell_style = cell_get_mstyle (cell);
-	if (cell->format != NULL &&
-	    !style_format_is_general (cell->format) &&
+	if (cell->value != NULL &&
+	    VALUE_FMT (cell->value) != NULL &&
+	    !style_format_is_general (VALUE_FMT (cell->value)) &&
 	    style_format_is_general (mstyle_get_format (cell_style))) {
 		cell_style = mstyle_copy (cell_style);
-		mstyle_set_format (cell_style, cell->format);
+		mstyle_set_format (cell_style, VALUE_FMT (cell->value));
 		c->xf = put_mstyle (esheet->wb, cell_style);
 		mstyle_ref (cell_style);
 	} else
@@ -1839,7 +1804,7 @@ gather_styles (ExcelWorkbook *wb)
 		s = g_ptr_array_index (wb->sheets, i);
 		n += g_hash_table_size (s->gnum_sheet->cell_hash);
 		n += s->max_row;
-	}	
+	}
 
 	count_io_progress_set (wb->io_context, n, N_CELLS_BETWEEN_UPDATES);
 	for (i = 0; i < wb->sheets->len; i++) {
@@ -2013,76 +1978,33 @@ border_type_to_excel (StyleBorderType btype, MsBiffVersion ver)
 }
 
 /**
- * fixup_fill_colors
- * @xfd  XF data
+ * style_color_to_pal_index
+ * @color color
+ * @wb    workbook
+ * @auto_back     Auto colors to compare against.
+ * @auto_font
  *
- * Do yucky stuff with fill foreground and background colors
- *
- * Solid fill patterns seem to reverse the meaning of foreground and
- * background
- *
- * FIXME:
- * Import side code does not flip colors if xfd->pat_foregnd_col == 0.
- *
- * This table shows import side behaviour when fill pattern is 1:
- *
- * bg(file) fg(file)    bg(internal) fg(internal)
- *  == 0     == 0         0            0
- *  == 0     != 0         fg(file)     0
- *  != 0     == 0         bg(file)     0
- *  != 0     != 0         fg(file)     bg(file)
- *
- * We can see from the table that bg(internal) is 0 only if fg(internal)
- * is also 0 .
- *
- * For export, the following two cases are obvious
- *  bg(internal)  fg(internal)     bg(file)  fg(file)
- *     == 0           == 0            0         0
- *     != 0           != 0        fg(internal)  bg(internal)
- * This one looks like we can choose, but testing with Excel showed that we
- * do have to reverse colors:
- *  bg(internal)  fg(internal)
- *     != 0           == 0
- *
- * Finally,  we have to do something special for bg(internal) == 0. In
- * this situation, I have seen Excel flip colors, but use 8 rather than 0
- * for black, i.e. fg(file) = 8, bg(file) = fg(internal). This is what
- * we'll do, although I don't know if Excel always does.
- *
- * This makes us compatible with our own import code.  The import
- * side test can't be correct, though. Excel displays fg(file) = 0,
- * bg(file) = 1 as black (=0) background. Gnumeric displays background
- * as white, since fg(file) = 0.  But I'll leave this ugliness in for
- * now.
- *
- * 2000-06-21 Jon Kre Hellan
- * A user reported that Excel wouldn't open the cell format dialog of cells
- * exported from Gnumeric. This was due to our handling of black.
- * We exported cells with black pattern foreground and white background,
- * unless something else had been explicitly selected. We used fg/bg 0/1
- * (PALETTE_BLACK/PALETTE_WHITE). Exporting border colors as palette index 0
- * resulted in the same problem. Excel renders the cells as we intend, but
- * refuses to open the cell format dialog.
- *
- * Excel apparently wants 8 (PALETTE_ALSO_BLACK) for black in foreground and
- * border. (Of course - most of the time it uses autocolors.) On the other
- * hand, cell format dialog can't be shown when 8 is used for background.
- * For fonts, it looks like Excel doesn't care.
- **/
-static void
-fixup_fill_colors (BiffXFData *xfd)
+ * Return Excel color index, possibly auto, for a style color.
+ * The auto colors are passed in by caller to avoid having to ref and unref
+ * the same autocolors over and over.
+ */
+static guint8
+style_color_to_pal_index (StyleColor *color, ExcelWorkbook *wb,
+			  StyleColor *auto_back, StyleColor *auto_font)
 {
-	guint8 c;
+	guint8 idx;
+	
+	if (color->is_auto) {
+		if (color == auto_back)
+			idx = PALETTE_AUTO_BACK;
+		else if (color == auto_font)
+				idx = PALETTE_AUTO_FONT;
+		else
+			idx = PALETTE_AUTO_PATTERN;
+	} else 
+		idx = palette_get_index	(wb, style_color_to_rgb888 (color));
 
-	if ((xfd->fill_pattern_idx == 1)
-	    && ((xfd->pat_foregnd_col != PALETTE_BLACK)
-		|| (xfd->pat_backgnd_col != PALETTE_BLACK))) {
-		c = xfd->pat_backgnd_col;
-		xfd->pat_backgnd_col = xfd->pat_foregnd_col;
-		xfd->pat_foregnd_col = c;
-	}
-	if (xfd->pat_foregnd_col == PALETTE_BLACK)
-		xfd->pat_foregnd_col = PALETTE_ALSO_BLACK;
+	return idx;
 }
 
 /**
@@ -2121,11 +2043,11 @@ get_xf_differences (ExcelWorkbook *wb, BiffXFData *xfd, MStyle *parentst)
 			break;
 		}
 	}
-	if (xfd->pat_foregnd_col != PALETTE_BLACK
-	    || xfd->pat_backgnd_col != PALETTE_WHITE
+	if (xfd->pat_foregnd_col != PALETTE_AUTO_PATTERN
+	    || (xfd->pat_backgnd_col) != PALETTE_AUTO_BACK
 	    || xfd->fill_pattern_idx != FILL_MAGIC)
 		xfd->differences |= 1 << MS_BIFF_D_FILL_BIT;
-	if (xfd->hidden || xfd->locked)
+	if (xfd->hidden || !xfd->locked)
 		xfd->differences |= 1 << MS_BIFF_D_LOCK_BIT;
 }
 
@@ -2177,12 +2099,9 @@ log_xf_data (ExcelWorkbook *wb, BiffXFData *xfd, int idx)
  * See S59E1E.HTM
  *
  * All BIFF V7 features are implemented, except:
- * - hidden and locked - not yet in gnumeric.
+ * - hidden - not yet in gnumeric.
  *
  * Apart from font, the style elements we retrieve do *not* need to be unrefed.
- *
- * FIXME:
- * It may be possible to recognize auto contrast for a few simple cases.
  **/
 static void
 build_xf_data (ExcelWorkbook *wb, BiffXFData *xfd, MStyle *st)
@@ -2192,9 +2111,8 @@ build_xf_data (ExcelWorkbook *wb, BiffXFData *xfd, MStyle *st)
 	int pat;
 	StyleColor *pattern_color;
 	StyleColor *back_color;
-	guint pattern_pal_color;
-	guint back_pal_color;
-	gint c;
+	StyleColor *auto_back = style_color_auto_back ();
+	StyleColor *auto_font = style_color_auto_font ();
 	int i;
 
 	memset (xfd, 0, sizeof *xfd);
@@ -2223,15 +2141,12 @@ build_xf_data (ExcelWorkbook *wb, BiffXFData *xfd, MStyle *st)
 		b = mstyle_get_border (st, MSTYLE_BORDER_TOP + i);
 		if (b) {
 			xfd->border_type[i] = b->line_type;
-			if (b->color) {
-				c = palette_get_index (
-					wb, style_color_to_int (b->color));
-				/* Excel doesn't want index 0 for black in
-				 * borders */
-				if (c == PALETTE_BLACK)
-					c = PALETTE_ALSO_BLACK;
-				xfd->border_color[i] = c;
-			}
+			xfd->border_color[i]
+				= b->color
+				? style_color_to_pal_index (b->color, wb,
+							    auto_back,
+							    auto_font)
+				: PALETTE_AUTO_PATTERN;
 		}
 	}
 
@@ -2240,23 +2155,28 @@ build_xf_data (ExcelWorkbook *wb, BiffXFData *xfd, MStyle *st)
 
 	pattern_color = mstyle_get_color (st, MSTYLE_COLOR_PATTERN);
 	back_color   = mstyle_get_color (st, MSTYLE_COLOR_BACK);
-	if (pattern_color) {
-		pattern_pal_color = style_color_to_int (pattern_color);
-	} else {
-		pattern_pal_color = PALETTE_BLACK;
-	}
-	if (back_color) {
-		back_pal_color = style_color_to_int (back_color);
-	} else {
-		back_pal_color = PALETTE_WHITE;
-	}
-	xfd->pat_backgnd_col = palette_get_index (wb, back_pal_color);
-	xfd->pat_foregnd_col = palette_get_index (wb, pattern_pal_color);
+	xfd->pat_foregnd_col
+		= pattern_color
+		? style_color_to_pal_index (pattern_color, wb, auto_back,
+					    auto_font)
+		: PALETTE_AUTO_PATTERN;
+	xfd->pat_backgnd_col
+		= back_color
+		? style_color_to_pal_index (back_color, wb, auto_back,
+					    auto_font)
+		: PALETTE_AUTO_BACK;
 
 	/* Solid patterns seem to reverse the meaning */
-	fixup_fill_colors (xfd);
+ 	if (xfd->fill_pattern_idx == FILL_SOLID) {
+		guint8 c = xfd->pat_backgnd_col;
+		xfd->pat_backgnd_col = xfd->pat_foregnd_col;
+		xfd->pat_foregnd_col = c;
+	}
 
 	get_xf_differences (wb, xfd, wb->xf->default_style);
+
+	style_color_unref (auto_font);
+	style_color_unref (auto_back);
 }
 
 /**
@@ -2341,6 +2261,7 @@ write_xf_record (BiffPut *bp, ExcelWorkbook *wb, BiffXFData *xfd)
 	guint8 data[256];
 	guint16 itmp;
 	int lp;
+	int btype;
 
 	for (lp = 0; lp < 250; lp++)
 		data[lp] = 0;
@@ -2378,36 +2299,46 @@ write_xf_record (BiffPut *bp, ExcelWorkbook *wb, BiffXFData *xfd)
 		itmp |= xfd->differences & 0xFC00; /* Difference bits */
 		MS_OLE_SET_GUINT16(data+6, itmp);
 
-		itmp = 1 << 13; /* fSxButton bit - apparently always set */
+		/* Documentation is wrong - there is no fSxButton bit.
+		 * The bg color uses the bit */
+		itmp = 0;
 		/* Fill pattern foreground color */
 		itmp |= xfd->pat_foregnd_col & 0x7f;
 		/* Fill pattern background color */
-		itmp |= (xfd->pat_backgnd_col << 7) & 0x1f80;
+		itmp |= (xfd->pat_backgnd_col << 7) & 0x3f80;
 		MS_OLE_SET_GUINT16(data+8, itmp);
 
 		itmp  = xfd->fill_pattern_idx & 0x3f;
 
 		/* Borders */
-		itmp |= (border_type_to_excel (xfd->border_type[STYLE_BOTTOM],
-					       wb->ver)
-			  << 6) & 0x1c0;
-		itmp |= (xfd->border_color[STYLE_BOTTOM] << 9) & 0xfe00;
+		btype = xfd->border_type[STYLE_BOTTOM];
+		if (btype != STYLE_BORDER_NONE) {
+			itmp |= (border_type_to_excel (btype, wb->ver) << 6)
+				& 0x1c0;
+			itmp |= (xfd->border_color[STYLE_BOTTOM] << 9)
+				& 0xfe00;
+		}
 		MS_OLE_SET_GUINT16(data+10, itmp);
 
-		itmp  = border_type_to_excel (xfd->border_type[STYLE_TOP],
-					      wb->ver)
-			& 0x7;
+		itmp  = 0;
+		btype = xfd->border_type[STYLE_TOP];
+		if (btype != STYLE_BORDER_NONE) {
+			itmp |= border_type_to_excel (btype, wb->ver) & 0x7;
+			itmp |= (xfd->border_color[STYLE_TOP] << 9) & 0xfe00;
+		}
 		itmp |= (border_type_to_excel (xfd->border_type[STYLE_LEFT],
 					       wb->ver)
 			 << 3) & 0x38;
 		itmp |= (border_type_to_excel (xfd->border_type[STYLE_RIGHT],
 					       wb->ver)
 			 << 6) & 0x1c0;
-		itmp |= (xfd->border_color[STYLE_TOP] << 9) & 0xfe00;
 		MS_OLE_SET_GUINT16(data+12, itmp);
 
-		itmp  = xfd->border_color[STYLE_LEFT] & 0x7f;
-		itmp |= (xfd->border_color[STYLE_RIGHT] << 7) & 0x3f80;
+		itmp  = 0;
+		if (xfd->border_type[STYLE_LEFT] != STYLE_BORDER_NONE)
+			itmp  |= xfd->border_color[STYLE_LEFT] & 0x7f;
+		if (xfd->border_type[STYLE_RIGHT] != STYLE_BORDER_NONE)
+			itmp |= (xfd->border_color[STYLE_RIGHT] << 7) & 0x3f80;
 		MS_OLE_SET_GUINT16(data+14, itmp);
 
 		ms_biff_put_var_write (bp, data, 16);
@@ -2554,8 +2485,10 @@ write_value (BiffPut *bp, Value *v, MsBiffVersion ver,
 			(((((int)val)<<2)>>2) == ((int)val));
 
 		d (3, printf ("Writing %g is (%g %g) is int ? %d\n",
-			      val, 1.0*(int)val,
-			      1.0*(val - (int)val), is_int););
+			      (double)val,
+			      (double)(1.0 * (int)val),
+			      (double)(1.0 * (val - (int)val)),
+			      is_int););
 
 		/* FIXME : Add test for double with 2 digits of fraction
 		 * and represent it as a mode 3 RK (val*100) construct */
@@ -2586,24 +2519,19 @@ write_value (BiffPut *bp, Value *v, MsBiffVersion ver,
 	}
 	case VALUE_STRING: {
 		guint8 data[16];
-		gint len;
-		char *buf;
-
 		g_return_if_fail (v->v_str.val->str);
 
 		g_return_if_fail (ver < MS_BIFF_V8); /* Use SST stuff in fulness of time */
 
 		/* See: S59DDC.HTM ( for RSTRING ) */
 		/* See: S59D9D.HTM ( for LABEL ) */
-		len = biff_convert_text(&buf, v->v_str.val->str, MS_BIFF_V7);
 		ms_biff_put_var_next   (bp, (0x200 | BIFF_LABEL));
 		EX_SETXF (data, xf);
 		EX_SETCOL(data, col);
 		EX_SETROW(data, row);
-		EX_SETSTRLEN (data, len);
+		EX_SETSTRLEN (data, strlen(v->v_str.val->str));
 		ms_biff_put_var_write  (bp, data, 8);
-		biff_put_text (bp, buf, len, MS_BIFF_V7, FALSE, AS_PER_VER);
-		g_free(buf);
+		biff_put_text (bp, v->v_str.val->str, MS_BIFF_V7, FALSE, AS_PER_VER);
 		ms_biff_put_commit (bp);
 		break;
 	}
@@ -2719,14 +2647,11 @@ write_formula (BiffPut *bp, ExcelSheet *esheet, const Cell *cell, gint16 xf)
 	}
 
 	if (string_result) {
-		gint len;
-		gchar *str, *buf;
+		gchar *str;
 
 		ms_biff_put_var_next (bp, 0x200|BIFF_STRING);
 		str = value_get_as_string (v);
-		len = biff_convert_text(&buf, str, MS_BIFF_V7);
-		biff_put_text (bp, buf, len, MS_BIFF_V7, TRUE, SIXTEEN_BIT);
-		g_free (buf);
+		biff_put_text (bp, str, MS_BIFF_V7, TRUE, SIXTEEN_BIT);
 		g_free (str);
 		ms_biff_put_commit (bp);
 	}
@@ -2751,13 +2676,11 @@ write_biff7_comments (BiffPut *bp, ExcelSheet *esheet)
 		Range const *pos = sheet_object_range_get (SHEET_OBJECT (cc));
 
 		guint16 len;
-		char *buf, *p;
 
 		g_return_if_fail (comment != NULL);
 		g_return_if_fail (pos != NULL);
 
-		len = biff_convert_text(&buf, comment, ver);
-		p = buf;
+		len = strlen (comment);
 		ms_biff_put_var_next (bp, BIFF_NOTE);
 		MS_OLE_SET_GUINT16 (data + 0, pos->start.row);
 		MS_OLE_SET_GUINT16 (data + 2, pos->start.col);
@@ -2766,11 +2689,15 @@ write_biff7_comments (BiffPut *bp, ExcelSheet *esheet)
 
 repeat:
 		if (len > MAX_BIFF_NOTE_CHUNK) {
-			biff_put_text (bp, p, MAX_BIFF_NOTE_CHUNK, ver, FALSE, AS_PER_VER);
+			char const tmp = comment [MAX_BIFF_NOTE_CHUNK];
+
+			comment [MAX_BIFF_NOTE_CHUNK] = '\0';
+			biff_put_text (bp, comment, ver, FALSE, AS_PER_VER);
+			comment [MAX_BIFF_NOTE_CHUNK] = tmp;
 
 			ms_biff_put_commit (bp);
 
-			p += MAX_BIFF_NOTE_CHUNK;
+			comment += MAX_BIFF_NOTE_CHUNK;
 			len -= MAX_BIFF_NOTE_CHUNK;
 
 			ms_biff_put_var_next (bp, BIFF_NOTE);
@@ -2781,10 +2708,9 @@ repeat:
 
 			goto repeat;
 		} else {
-			biff_put_text (bp, p, len, ver, FALSE, AS_PER_VER);
+			biff_put_text (bp, comment, ver, FALSE, AS_PER_VER);
 			ms_biff_put_commit (bp);
 		}
-		g_free(buf);
 	}
 	g_slist_free (comments);
 }
@@ -3006,24 +2932,24 @@ init_base_char_width_for_write (ExcelSheet *esheet)
 {
 	ExcelFont *f = NULL;
 	/* default to Arial 10 */
-	char *name = "Arial";
-	double size = 20.* 10.;
+	const char *name = "Arial";
+	double size_pts = 20.* 10.;
 
 	if (esheet && esheet->wb
 	    && esheet->wb->xf && esheet->wb->xf->default_style) {
 		f = excel_font_new (esheet->wb->xf->default_style);
 		if (f) {
 			name = f->style_font->font_name;
-			size = f->style_font->size * 20.;
+			size_pts = f->style_font->size_pts * 20.;
 			excel_font_free (f);
 		}
 	}
 
-	d (1, printf ("Font for column sizing: %s %.1f\n", name, size););
+	d (1, printf ("Font for column sizing: %s %.1f\n", name, size_pts););
 	esheet->base_char_width =
-		lookup_font_base_char_width_new (name, size, FALSE);
+		lookup_font_base_char_width_new (name, size_pts, FALSE);
 	esheet->base_char_width_default =
-		lookup_font_base_char_width_new (name, size, TRUE);
+		lookup_font_base_char_width_new (name, size_pts, TRUE);
 }
 
 /**
@@ -3086,7 +3012,7 @@ write_default_col_width (BiffPut *bp, ExcelSheet *esheet)
 /**
  * write_colinfo
  * @bp:   BIFF buffer
- * @esheet: 
+ * @esheet:
  * @ci   : the descriptor of the first col
  * @last_index : the index of the last contiguous identical col
  * @xf_index   : the style index to the entire col (< 0 for none)
@@ -3404,7 +3330,7 @@ write_rowinfo (BiffPut *bp, ExcelSheet *esheet, guint32 row, guint32 last_col)
 /**
  * write_db_cell
  * @bp        BIFF buffer
- * @esheet    
+ * @esheet
  * @ri_start  start positions of first 2 rowinfo records
  * @rc_start  start positions of first row in each cell in block
  * @nrows  no. of rows in block.
@@ -3574,7 +3500,7 @@ excel_sheet_new (ExcelWorkbook *ewb, Sheet *gnum_sheet, IOContext *context)
 	/* Ignore spans and merges past the bound */
 	extent = sheet_get_extent (gnum_sheet, FALSE);
 
-	if (extent.end.row > maxrows) {
+	if (extent.end.col > maxrows) {
 		char *msg = g_strdup_printf (
 			_("Too many rows for this format (%d > %d)"),
 			  extent.end.col, maxrows);
@@ -3785,7 +3711,7 @@ ms_excel_write_workbook (IOContext *context, MsOle *file, void *state,
                          MsBiffVersion ver)
 {
 	MsOleErr     result;
-	char        *strname;
+	const char  *strname;
 	MsOleStream *str;
 	BiffPut     *bp;
 	ExcelWorkbook *wb = state;
