@@ -122,7 +122,7 @@ html_get_text_color (Cell *cell, MStyle *mstyle, guint *r, guint *g, guint *b)
 {
 	StyleColor *textColor;
 
-	textColor= cell_get_render_color (cell);
+	textColor = cell_get_render_color (cell);
 	if (textColor == NULL)
 		textColor = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
 
@@ -221,7 +221,7 @@ write_cell (FILE *fp, Sheet *sheet, gint row, gint col, html_version_t version)
 			fputs(" valign=\"bottom\" ", fp);
 			break;
 		case VALIGN_CENTER:
-			fputs(" valign=\"middle\" ", fp);
+			fputs(" valign=\"center\" ", fp);
 			break;
 		case VALIGN_JUSTIFY:
 			fputs(" valign=\"baseline\" ", fp);
