@@ -6,9 +6,8 @@
  * Copyright (C) 2000 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -133,14 +132,6 @@ xml_sax_attr_int (xmlChar const * const *attrs, char const *name, int *res)
 	return TRUE;
 }
 
-static gboolean
-xml_sax_int (xmlChar const *chars, int *res)
-{
-	char *end;
-	*res = strtol ((gchar *)chars, &end, 10);
-	return *end == '\0';
-}
-
 static gboolean
 xml_sax_attr_cellpos (xmlChar const * const *attrs, char const *name, CellPos *val)
 {
@@ -437,9 +428,7 @@ typedef struct _XMLSaxParseState
 	struct {
 		char *name;
 		char *value;
-		int   type;
 	} attribute;
-	GList *attributes;
 
 	/* Only valid when parsing wb or sheet names */
 	struct {
@@ -466,7 +455,7 @@ typedef struct _XMLSaxParseState
 	CellPos cell;
 	int expr_id, array_rows, array_cols;
 	int value_type;
-	unsigned char const *value_fmt;
+	StyleFormat *value_fmt;
 
 	GString *content;
 
@@ -591,79 +580,17 @@ xml_sax_wb_view (XMLSaxParseState *state, xmlChar const **attrs)
 		wb_view_preferred_size (state->wb_view, width, height);
 }
 
-static void
-xml_sax_arg_set (GtkArg *arg, gchar *string)
-{
-	switch (arg->type) {
-	case GTK_TYPE_CHAR:
-		GTK_VALUE_CHAR (*arg) = string[0];
-		break;
-	case GTK_TYPE_UCHAR:
-		GTK_VALUE_UCHAR (*arg) = string[0];
-		break;
-	case GTK_TYPE_BOOL:
-		if (!strcmp (string, "TRUE"))
-			GTK_VALUE_BOOL (*arg) = TRUE;
-		else
-			GTK_VALUE_BOOL (*arg) = FALSE;
-		break;
-	case GTK_TYPE_INT:
-		GTK_VALUE_INT (*arg) = atoi (string);
-		break;
-	case GTK_TYPE_UINT:
-		GTK_VALUE_UINT (*arg) = atoi (string);
-		break;
-	case GTK_TYPE_LONG:
-		GTK_VALUE_LONG (*arg) = atol (string);
-		break;
-	case GTK_TYPE_ULONG:
-		GTK_VALUE_ULONG (*arg) = atol (string);
-		break;
-	case GTK_TYPE_FLOAT:
-		GTK_VALUE_FLOAT (*arg) = atof (string);
-		break;
-	case GTK_TYPE_DOUBLE:
-		GTK_VALUE_DOUBLE (*arg) = atof (string);
-		break;
-	case GTK_TYPE_STRING:
-		GTK_VALUE_STRING (*arg) = g_strdup (string);
-		break;
-	}
-}
-
 static void
 xml_sax_finish_parse_wb_attr (XMLSaxParseState *state)
 {
-	GtkArg *arg;
-
 	g_return_if_fail (state->attribute.name != NULL);
 	g_return_if_fail (state->attribute.value != NULL);
-	g_return_if_fail (state->attribute.type >= 0);
 
-	arg = gtk_arg_new (state->attribute.type);
-	arg->name = state->attribute.name;
-	xml_sax_arg_set (arg, state->attribute.value);
-	state->attributes = g_list_prepend (state->attributes, arg);
+	wb_view_set_attribute (state->wb_view,
+		state->attribute.name, state->attribute.value);
 
-	state->attribute.type = -1;
-	g_free (state->attribute.value);
-	state->attribute.value = NULL;
-	state->attribute.name = NULL;
-}
-
-static void
-xml_sax_free_arg_list (GList *start)
-{
-	GList *list = start;
-	while (list) {
-		GtkArg *arg = list->data;
-		if (arg) {
-			g_free (arg->name);
-			gtk_arg_free (arg, FALSE);
-		}
-		list = list->next;
-	}
-	g_list_free (start);
+	g_free (state->attribute.value);	state->attribute.value = NULL;
+	g_free (state->attribute.name);		state->attribute.name = NULL;
 }
 
 static void
@@ -683,13 +610,8 @@ xml_sax_attr_elem (XMLSaxParseState *state)
 		state->attribute.value = g_strndup (content, len);
 		break;
 
-	case STATE_WB_ATTRIBUTES_ELEM_TYPE :
-	{
-		int type;
-		if (xml_sax_int ((xmlChar *)content, &type))
-			state->attribute.type = type;
+	case STATE_WB_ATTRIBUTES_ELEM_TYPE : /* ignore */
 		break;
-	}
 
 	default :
 		g_assert_not_reached ();
@@ -763,7 +685,7 @@ xml_sax_sheet_name (XMLSaxParseState *state)
 		state->sheet = sheet_new (state->wb, content);
 		workbook_sheet_attach (state->wb, state->sheet, NULL);
 	}
-	
+
 	if (state->display_formulas >= 0)
 		state->sheet->display_formulas = state->display_formulas;
 	if (state->hide_zero >= 0)
@@ -1195,7 +1117,7 @@ xml_sax_validation (XMLSaxParseState *state, xmlChar const **attrs)
 		} else if (!strcmp (attrs[0], "Title")) {
 			state->validation.title = g_strdup ((gchar *)attrs[1]);
 		} else if (!strcmp (attrs[0], "Message")) {
-			state->validation.msg = g_strdup ((gchar *)attrs[1]);			
+			state->validation.msg = g_strdup ((gchar *)attrs[1]);
 		} else if (xml_sax_attr_bool (attrs, "AllowBlank", &b_dummy)) {
 			state->validation.allow_blank = b_dummy;
 		} else if (xml_sax_attr_bool (attrs, "UseDropdown", &b_dummy)) {
@@ -1281,7 +1203,7 @@ xml_sax_cell (XMLSaxParseState *state, xmlChar const **attrs)
 	int row = -1, col = -1;
 	int rows = -1, cols = -1;
 	int value_type = -1;
-	unsigned char const *value_fmt = NULL;
+	StyleFormat *value_fmt = NULL;
 	int expr_id = -1;
 
 	g_return_if_fail (state->cell.row == -1);
@@ -1299,7 +1221,7 @@ xml_sax_cell (XMLSaxParseState *state, xmlChar const **attrs)
 		else if (xml_sax_attr_int (attrs, "ExprID", &expr_id)) ;
 		else if (xml_sax_attr_int (attrs, "ValueType", &value_type)) ;
 		else if (!strcmp (attrs[0], "ValueFormat"))
-			value_fmt = attrs[1];
+			value_fmt = style_format_new_XL ((char *)attrs[1], FALSE);
 		else
 			xml_sax_unknown_attr (state, attrs, "Cell");
 	}
@@ -1407,7 +1329,7 @@ xml_sax_cell_content (XMLSaxParseState *state)
 	int const array_rows = state->array_rows;
 	int const expr_id = state->expr_id;
 	int const value_type = state->value_type;
-	unsigned char const *value_fmt =state->value_fmt;
+	StyleFormat *value_fmt = state->value_fmt;
 	gpointer const id = GINT_TO_POINTER (expr_id);
 	gpointer expr = NULL;
 
@@ -1444,11 +1366,8 @@ xml_sax_cell_content (XMLSaxParseState *state)
 		} else if (state->version >= GNUM_XML_V3 ||
 			   xml_not_used_old_array_spec (cell, content)) {
 			if (value_type > 0) {
-				Value *v = value_new_from_string (value_type, content);
-				StyleFormat *sf = (value_fmt != NULL)
-					? style_format_new_XL ((char *)value_fmt, FALSE)
-					: NULL;
-				cell_set_value (cell, v, sf);
+				Value *v = value_new_from_string (value_type, content, value_fmt);
+				cell_set_value (cell, v);
 			} else
 				cell_set_text (cell, content);
 		}
@@ -1471,7 +1390,7 @@ xml_sax_cell_content (XMLSaxParseState *state)
 			GINT_TO_POINTER (expr_id));
 
 		if (expr != NULL)
-			cell_set_expr (cell, expr, NULL);
+			cell_set_expr (cell, expr);
 		else
 			g_warning ("XML-IO : Missing shared expression");
 	} else if (is_new_cell)
@@ -1480,7 +1399,10 @@ xml_sax_cell_content (XMLSaxParseState *state)
 		 * If it was created by a previous array
 		 * we do not want to erase it.
 		 */
-		cell_set_value (cell, value_new_empty (), NULL);
+		cell_set_value (cell, value_new_empty ());
+
+	if (value_fmt != NULL)
+		style_format_unref (value_fmt);
 }
 
 static void
@@ -1496,7 +1418,7 @@ xml_sax_merge (XMLSaxParseState *state)
 static void
 xml_sax_object (XMLSaxParseState *state, xmlChar const **attrs)
 {
-	
+
 }
 
 static void
@@ -1504,11 +1426,11 @@ xml_sax_finish_parse_wb_names_name (XMLSaxParseState *state)
 {
 	g_return_if_fail (state->name.name != NULL);
 	g_return_if_fail (state->name.value != NULL);
-	
+
 	if (state->version >= GNUM_XML_V7) {
 		ParseError  perr;
 		ParsePos    pos;
-		
+
 		parse_pos_init (&pos, NULL, state->sheet, 0, 0);
 		if (state->name.position) {
 			CellRef tmp;
@@ -1551,7 +1473,7 @@ xml_sax_finish_parse_sheet_names_name (XMLSaxParseState *state)
 {
 	ParseError  perr;
 	ParsePos    pos;
-	
+
 	g_return_if_fail (state->name.name != NULL);
 	g_return_if_fail (state->name.value != NULL);
 
@@ -1697,7 +1619,7 @@ xml_sax_start_element (XMLSaxParseState *state, xmlChar const *name, xmlChar con
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_WB_SHEETS :
 		if (xml_sax_switch_state (state, name, STATE_SHEET)) {
 			xml_sax_sheet_start (state, attrs);
@@ -1728,7 +1650,7 @@ xml_sax_start_element (XMLSaxParseState *state, xmlChar const *name, xmlChar con
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_SHEET_NAMES:
 		if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME)) {
 		} else
@@ -1741,7 +1663,7 @@ xml_sax_start_element (XMLSaxParseState *state, xmlChar const *name, xmlChar con
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_SHEET_MERGED_REGION :
 		if (!xml_sax_switch_state (state, name, STATE_SHEET_MERGE))
 			xml_sax_unknown_state (state, name);
@@ -1823,14 +1745,14 @@ xml_sax_start_element (XMLSaxParseState *state, xmlChar const *name, xmlChar con
 		else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_STYLE_VALIDATION :
 		if (xml_sax_switch_state (state, name, STATE_STYLE_VALIDATION_EXPR0));
 		else if (xml_sax_switch_state (state, name, STATE_STYLE_VALIDATION_EXPR1));
 		else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_SHEET_COLS :
 		if (xml_sax_switch_state (state, name, STATE_COL))
 			xml_sax_colrow (state, attrs, TRUE);
@@ -1882,7 +1804,7 @@ xml_sax_start_element (XMLSaxParseState *state, xmlChar const *name, xmlChar con
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
-		
+
 	case STATE_NAMES:
 		if (xml_sax_switch_state (state, name, STATE_NAMES_NAME)) {
 		} else
@@ -1921,12 +1843,6 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		xml_sax_finish_parse_wb_attr (state);
 		break;
 
-	case STATE_WB_ATTRIBUTES :
-		wb_view_set_attribute_list (state->wb_view, state->attributes);
-		xml_sax_free_arg_list (state->attributes);
-		state->attributes = NULL;
-		break;
-
 	case STATE_SHEET_SELECTIONS :
 		xml_sax_selection_end (state);
 		break;
@@ -1941,7 +1857,6 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		break;
 
 	case STATE_WB_ATTRIBUTES_ELEM_NAME :
-	case STATE_WB_ATTRIBUTES_ELEM_TYPE :
 	case STATE_WB_ATTRIBUTES_ELEM_VALUE :
 		xml_sax_attr_elem (state);
 		g_string_truncate (state->content, 0);
@@ -1957,7 +1872,7 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		xml_sax_wb_sheetname (state);
 		g_string_truncate (state->content, 0);
 		break;
-		
+
 	case STATE_SHEET_NAME :
 		xml_sax_sheet_name (state);
 		g_string_truncate (state->content, 0);
@@ -1967,7 +1882,7 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		xml_sax_sheet_zoom (state);
 		g_string_truncate (state->content, 0);
 		break;
-		
+
 	case STATE_SHEET_NAMES_NAME :
 		xml_sax_finish_parse_sheet_names_name (state);
 		break;
@@ -1977,7 +1892,7 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		xml_sax_name (state);
 		g_string_truncate (state->content, 0);
 		break;
-		
+
 	case STATE_PRINT_MARGIN_TOP :
 	case STATE_PRINT_MARGIN_BOTTOM :
 	case STATE_PRINT_MARGIN_LEFT :
@@ -1994,7 +1909,7 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 	case STATE_STYLE_VALIDATION :
 		xml_sax_validation_end (state);
 		break;
-		
+
 	case STATE_STYLE_VALIDATION_EXPR0 :
 	case STATE_STYLE_VALIDATION_EXPR1 :
 		xml_sax_validation_expr_end (state);
@@ -2018,7 +1933,7 @@ xml_sax_end_element (XMLSaxParseState *state, const xmlChar *name)
 		xml_sax_merge (state);
 		g_string_truncate (state->content, 0);
 		break;
-		
+
 	case STATE_NAMES_NAME :
 		xml_sax_finish_parse_wb_names_name (state);
 		break;
@@ -2042,7 +1957,6 @@ xml_sax_characters (XMLSaxParseState *state, const xmlChar *chars, int len)
 {
 	switch (state->state) {
 	case STATE_WB_ATTRIBUTES_ELEM_NAME :
-	case STATE_WB_ATTRIBUTES_ELEM_TYPE :
 	case STATE_WB_ATTRIBUTES_ELEM_VALUE :
 	case STATE_WB_SUMMARY_ITEM_NAME :
 	case STATE_WB_SUMMARY_ITEM_VALUE_INT :
@@ -2099,8 +2013,6 @@ xml_sax_start_document (XMLSaxParseState *state)
 	state->content = g_string_sized_new (128);
 
 	state->attribute.name = state->attribute.value = NULL;
-	state->attribute.type = -1;
-	state->attributes = NULL;
 
 	state->name.name = state->name.value = state->name.position = NULL;
 
@@ -2175,7 +2087,7 @@ xml_sax_file_open (GnumFileOpener const *fo, IOContext *io_context,
 	 * and using vfs.
 	 */
 	ctxt = xmlCreateFileParserCtxt (filename);
-	if (ctxt == NULL) { 
+	if (ctxt == NULL) {
 		gnumeric_io_error_string (io_context, _("xmlCreateFileParserCtxt () failed."));
 		return;
 	}
