@@ -6,9 +6,13 @@
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
  */
 
-#include <config.h>
+#include <gnumeric-config.h>
+#include "gnumeric.h"
 #include "collect.h"
+
 #include "func.h"
+#include "value.h"
+#include "expr.h"
 #include "datetime.h"
 
 /* ------------------------------------------------------------------------- */
@@ -115,7 +119,7 @@ callback_function_collect (const EvalPos *ep, Value *value, void *closure)
  * gnum_float.
  */
 static gnum_float *
-collect_floats (GList *exprlist, const EvalPos *ep, CollectFlags flags,
+collect_floats (ExprList *exprlist, const EvalPos *ep, CollectFlags flags,
 		int *n, Value **error)
 {
 	Value * err;
@@ -127,14 +131,13 @@ collect_floats (GList *exprlist, const EvalPos *ep, CollectFlags flags,
 	cl.flags = flags;
 
 	err = function_iterate_argument_values (ep, &callback_function_collect,
-						&cl, exprlist, TRUE,
-						flags&COLLECT_IGNORE_BLANKS);
+		&cl, exprlist,
+		TRUE, flags&COLLECT_IGNORE_BLANKS);
 
 	if (err) {
 		g_assert (err->type == VALUE_ERROR);
 		g_free (cl.data);
-		/* Be careful not to make value_terminate into a real value */
-		*error = (err != value_terminate ())? value_duplicate (err) : err;
+		*error = err;
 		return NULL;
 	}
 
@@ -150,17 +153,17 @@ gnum_float *
 collect_floats_value (const Value *val, const EvalPos *ep,
 		      CollectFlags flags, int *n, Value **error)
 {
-	GList *exprlist;
+	ExprList *exprlist;
 	ExprTree *expr_val;
 	gnum_float *res;
 
 	expr_val = expr_tree_new_constant (value_duplicate (val));
-	exprlist = g_list_prepend (NULL, expr_val);
+	exprlist = expr_list_prepend (NULL, expr_val);
 
 	res = collect_floats (exprlist, ep, flags, n, error);
 
 	expr_tree_unref (expr_val);
-	g_list_free (exprlist);
+	expr_list_free (exprlist);
 
 	return res;
 }
@@ -169,7 +172,7 @@ collect_floats_value (const Value *val, const EvalPos *ep,
 /* ------------------------------------------------------------------------- */
 
 Value *
-float_range_function (GList *exprlist, FunctionEvalInfo *ei,
+float_range_function (ExprList *exprlist, FunctionEvalInfo *ei,
 		      float_range_function_t func,
 		      CollectFlags flags,
 		      char const *func_error)
