@@ -5,6 +5,7 @@
  *   Miguel de Icaza (miguel@gnu.org)
  *   Morten Welinder <terra@diku.dk>
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
+ *   Andreas J. Guelzow <aguelzow@taliesin.ca>
  */
 
 #include <gnumeric-config.h>
@@ -27,7 +28,7 @@ static int date_origin = 0;
  * The serial number of 19000228.  Excel allocates a serial number for
  * the non-existing date 19000229.
  */
-static const int date_serial_19000228 = 59;
+static int const date_serial_19000228 = 59;
 
 static void
 date_init (void)
@@ -69,16 +70,28 @@ datetime_serial_to_g (int serial)
 
 /* ------------------------------------------------------------------------- */
 
+/**
+ * Free GDate. Can be called with NULL without complaining.
+ */
+void
+datetime_g_free (GDate *d)
+{
+	if (d != NULL)
+		g_date_free (d);
+}
+
+/* ------------------------------------------------------------------------- */
+
 gnum_float
-datetime_value_to_serial_raw (const Value *v)
+datetime_value_to_serial_raw (Value const *v)
 {
 	gnum_float serial;
 
 	if (VALUE_IS_NUMBER (v))
 		serial = value_get_as_float (v);
 	else {
-		const char *str = value_peek_string (v);
-		Value *conversion = format_match (str, NULL, NULL);
+		char const *str = value_peek_string (v);
+		Value *conversion = format_match (str, NULL);
 
 		if (conversion) {
 			if (VALUE_IS_NUMBER (conversion))
@@ -104,7 +117,7 @@ datetime_timet_to_serial_raw (time_t t)
         g_date_clear (&date, 1);
 	g_date_set_time (&date, t);
 	secs = tm->tm_hour * 3600 + tm->tm_min * 60 + tm->tm_sec;
-	return datetime_g_to_serial (&date) + secs / (double)SECS_PER_DAY;
+	return datetime_g_to_serial (&date) + secs / (gnum_float)SECS_PER_DAY;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -118,7 +131,7 @@ datetime_serial_raw_to_serial (gnum_float raw)
 /* ------------------------------------------------------------------------- */
 
 int
-datetime_value_to_serial (const Value *v)
+datetime_value_to_serial (Value const *v)
 {
 	return datetime_serial_raw_to_serial (datetime_value_to_serial_raw (v));
 }
@@ -151,7 +164,7 @@ datetime_serial_to_timet (int serial)
 /* ------------------------------------------------------------------------- */
 
 GDate *
-datetime_value_to_g (const Value *v)
+datetime_value_to_g (Value const *v)
 {
 	int serial = datetime_value_to_serial (v);
 	return serial ? datetime_serial_to_g (serial) : NULL;
@@ -171,7 +184,7 @@ datetime_serial_raw_to_seconds (gnum_float raw)
 /* ------------------------------------------------------------------------- */
 
 int
-datetime_value_to_seconds (const Value *v)
+datetime_value_to_seconds (Value const *v)
 {
 	return datetime_serial_raw_to_seconds (datetime_value_to_serial_raw (v));
 }
@@ -266,7 +279,7 @@ datetime_isoweeknum (GDate *date)
 		if (nextjan1wday <= G_DATE_THURSDAY)
 			week = 1;
 	}
-	
+
 	return week;
 }
 
@@ -307,117 +320,110 @@ datetime_weeknum (GDate *date, int method)
 
 /* ------------------------------------------------------------------------- */
 
-/*
- * adjust_dates_basis
- *
- * @from      : GDate *
- * @to        : GDate * 
- * @basis     : int
- *	   "0  US 30/360\n"
- *         "1  actual days/actual days\n"
- *         "2  actual days/360\n"
- *         "3  actual days/365\n"
- *         "4  European 30/360\n"
- *
- * returns    : nothing. As side effects possibly adjusts from and to date
- *
- *
- */
-
-void
-adjust_dates_basis (GDate *from, GDate *to, int basis)
+static gint32
+days_between_BASIS_MSRB_30_360 (GDate *from, GDate *to)
 {
-	switch (basis) {
-	case BASIS_30Ep360:
-		if (g_date_day(to) == 31)
-			g_date_set_day (to, 30);
-		break;
-	case BASIS_30_360:
-		if (g_date_day(from) >= 30) {
-			g_date_set_day (from, 30);
-			if (g_date_day(to) == 31)
-				g_date_set_day (to, 30);
-		}
-		break;
-	case BASIS_30E360:
-		if (g_date_day(from) == 31)
-			g_date_set_day (from, 30);
-		if (g_date_day(to) == 31)
-			g_date_set_day (to, 30);
-		break;
-	default:
-		break;
+	int y1, m1, d1, y2, m2, d2;
+
+	y1 = g_date_year (from);
+	m1 = g_date_month (from);
+	d1 = g_date_day (from);
+	y2 = g_date_year (to);
+	m2 = g_date_month (to);
+	d2 = g_date_day (to);
+
+	if (d1 >= 30) {
+		d1 = 30;
+		if (d2 == 31)
+			d2 = 30;
 	}
-	return;
+
+	return (y2 - y1) * 360 + (m2 - m1) * 30 + (d2 - d1);
 }
 
-/* ------------------------------------------------------------------------- */
+static gint32
+days_between_BASIS_30E_360 (GDate *from, GDate *to)
+{
+	int y1, m1, d1, y2, m2, d2;
+
+	y1 = g_date_year (from);
+	m1 = g_date_month (from);
+	d1 = g_date_day (from);
+	y2 = g_date_year (to);
+	m2 = g_date_month (to);
+	d2 = g_date_day (to);
+
+	if (d1 == 31)
+		d1 = 30;
+	if (d2 == 31)
+		d2 = 30;
+
+	return (y2 - y1) * 360 + (m2 - m1) * 30 + (d2 - d1);
+}
+
+static gint32
+days_between_BASIS_30Ep_360 (GDate *from, GDate *to)
+{
+	int y1, m1, d1, y2, m2, d2;
+
+	y1 = g_date_year (from);
+	m1 = g_date_month (from);
+	d1 = g_date_day (from);
+	y2 = g_date_year (to);
+	m2 = g_date_month (to);
+	d2 = g_date_day (to);
+
+	if (d1 == 31)
+		d1 = 30;
+	if (d2 == 31) {
+		d2 = 1;
+		m2++;
+		/* No need to check for m2 == 13 since 12*30 == 360 */
+	}
+
+	return (y2 - y1) * 360 + (m2 - m1) * 30 + (d2 - d1);
+}
 
 /*
- * days_between_dep_basis
+ * days_between_basis
  *
  * @from      : GDate *
- * @to        : GDate * 
- * @basis     : int
- *	   "0  US 30/360\n"
- *         "1  actual days/actual days\n"
- *         "2  actual days/360\n"
- *         "3  actual days/365\n"
- *         "4  European 30/360\n"
+ * @to        : GDate *
+ * @basis     : basis_t
+ * see datetime.h and doc/fn-financial-basis.txt for details
+ *
+ * @in_order  : dates are considered in order
  *
  * returns    : Number of days strictly between from and to +1
  *
- * Note: before calling this function adjust_dates_basis _must_
- *       have been called.
  */
 
 gint32
-days_between_dep_basis (GDate *from, GDate *to, int basis)
+days_between_basis (GDate *from, GDate *to, int basis)
 {
-	GDate      from_date;
-	gint32     days;
-	gint       years;
-
 	switch (g_date_compare (from, to)) {
 	case 1:
-		return (- days_between_dep_basis (to, from, basis));
-	case 0:
-		return 0;
+		return (- days_between_basis (to, from, basis));
 	default:
 		break;
-	} 
+	}
 
-	if (basis == BASIS_ACTACT) 
+	switch (basis) {
+	case BASIS_ACT_ACT:
+	case BASIS_ACT_360:
+	case BASIS_ACT_365:
 		return (g_date_julian (to) - g_date_julian (from));
-
-	g_date_clear (&from_date, 1);
-	g_date_set_julian (&from_date, g_date_julian (from));
-	years = g_date_year(to) - g_date_year(from);
-	g_date_add_years (&from_date, years);
-
-	if ((basis == BASIS_ACT365 || basis == BASIS_ACT360) &&
-	    g_date_compare (&from_date, to) >= 0) {
-		years -= 1;
-		g_date_set_julian (&from_date, g_date_julian (from));
-		g_date_add_years (&from_date, years);
-	}
-
-	switch (basis) {
-	case BASIS_ACT365:
-		days = years * 365;
 		break;
-	default:
-		days = years * 360;
+	case BASIS_30E_360:
+		return days_between_BASIS_30E_360 (from, to);
 		break;
-	}
-
-	switch (basis) {
-	case BASIS_ACT360:
-	case BASIS_ACT365:
-		return days + g_date_julian (to) - g_date_julian (&from_date);
+	case BASIS_30Ep_360:
+		return days_between_BASIS_30Ep_360 (from, to);
+		break;
+	case BASIS_MSRB_30_360:
 	default:
-		return (days + 30 * (g_date_month(to) - g_date_month(&from_date))
-			+ g_date_day(to) - g_date_day(&from_date));
+		return days_between_BASIS_MSRB_30_360 (from, to);
+		break;
 	}
 }
 
@@ -429,7 +435,7 @@ days_between_dep_basis (GDate *from, GDate *to, int basis)
  * @settlement: GDate *
  * @maturity  : GDate *  must follow settlement strictly
  * @freq      : int      divides 12 evenly
- * @oem       : gboolean whether to do special end of month 
+ * @eom       : gboolean whether to do special end of month
  *                       handling
  * @next      : gboolean whether next or previous date
  *
@@ -439,16 +445,16 @@ days_between_dep_basis (GDate *from, GDate *to, int basis)
  */
 
 GDate *
-coup_cd (GDate *settlement, GDate *maturity, int freq, gboolean oem, gboolean next)
+coup_cd (GDate *settlement, GDate *maturity, int freq, gboolean eom, gboolean next)
 {
         int        months, periods;
 	GDate      *result;
-	gboolean   is_oem_special;
+	gboolean   is_eom_special;
 
-	is_oem_special = oem && g_date_is_last_of_month (maturity);
+	is_eom_special = eom && g_date_is_last_of_month (maturity);
 
 	months = 12 / freq;
-	periods = (g_date_year(maturity) - g_date_year(settlement));
+	periods = (g_date_year(maturity) - g_date_year (settlement));
 	if (periods > 0)
 		periods = (periods - 1) * freq;
 
@@ -458,65 +464,24 @@ coup_cd (GDate *settlement, GDate *maturity, int freq, gboolean oem, gboolean ne
 		g_date_set_julian (result, g_date_julian (maturity));
 		periods++;
 		g_date_subtract_months (result, periods * months);
-		if (is_oem_special) 
-			while (!g_date_is_last_of_month (result))
-				g_date_add_days (result, 1);
+		if (is_eom_special) {
+			int ndays = g_date_get_days_in_month
+				(g_date_get_month (result),
+				 g_date_get_year (result));
+			g_date_set_day (result, ndays);
+		}
 	} while (g_date_compare (settlement, result) < 0 );
 
 	if (next) {
 		g_date_set_julian (result, g_date_julian (maturity));
 		periods--;
 		g_date_subtract_months (result, periods * months);
-		if (is_oem_special) 
-			while (!g_date_is_last_of_month (result))
-				g_date_add_days (result, 1);
-	}
-
-	return result;
-}
-
-/* ------------------------------------------------------------------------- */
-
-/*
- * coup_cd_xl
- *
- * @settlement: GDate *
- * @maturity  : GDate *  must follow settlement strictly
- * @freq      : int      divides 12 evenly
- * @oem       : gboolean whether to do special end of month 
- *                       handling
- * @next      : gboolean whether next or previous date
- *
- * returns    : GDate *  date as returned by XL
- *
- * this function does not depend on the basis of counting!
- */
-
-GDate *
-coup_cd_xl (GDate *settlement, GDate *maturity, int freq, gboolean next)
-{
-	GDate      *result;
-	GDate      coupon;
-	gboolean   is_oem_special;
-	int        months = 12 / freq;
-
-	is_oem_special = g_date_is_last_of_month (maturity);
-
-	result = g_date_new();
-
-	g_date_clear (&coupon, 1);
-	g_date_set_julian (&coupon, g_date_julian (maturity));
-
-	do {
-		g_date_set_julian (result, g_date_julian (&coupon));
-		g_date_subtract_months (&coupon, months);
-		if (is_oem_special) 
-			while (!g_date_is_last_of_month (&coupon))
-				g_date_add_days (&coupon, 1);
-	} while (g_date_compare (settlement, &coupon) < 0 );
-
-	if (!next) {
-		g_date_set_julian (result, g_date_julian (&coupon));
+		if (is_eom_special) {
+			int ndays = g_date_get_days_in_month
+				(g_date_get_month (result),
+				 g_date_get_year (result));
+			g_date_set_day (result, ndays);
+		}
 	}
 
 	return result;
@@ -529,34 +494,30 @@ coup_cd_xl (GDate *settlement, GDate *maturity, int freq, gboolean next)
  * Returns the number of days in the coupon period of the settlement date.
  * Currently, returns negative numbers if the branch is not implemented.
  */
-int
-coupdays (GDate *settlement, GDate *maturity, int freq, int basis, gboolean oem, 
-	  gboolean xl)
+gnum_float
+coupdays (GDate *settlement, GDate *maturity, int freq, basis_t basis, gboolean eom)
 {
 	GDate *prev;
 	GDate *next;
-	int   days;
+	gint32   days;
 
         switch (basis) {
-        case BASIS_30Ep360:
-        case BASIS_30E360:
-        case BASIS_30_360:
-		return (12 / freq) * 30;
+	case BASIS_MSRB_30_360:
+	case BASIS_ACT_360:
+        case BASIS_30E_360:
+        case BASIS_30Ep_360:
+		return 360 / freq;
+	case BASIS_ACT_365:
+		return 365.0 / freq;
+	case BASIS_ACT_ACT:
 	default:
-		break;
+		next = coup_cd (settlement, maturity, freq, eom, TRUE);
+		prev = coup_cd (settlement, maturity, freq, eom, FALSE);
+		days = days_between_basis (prev, next, BASIS_ACT_ACT);
+		datetime_g_free (prev);
+		datetime_g_free (next);
+		return days;
         }
-
-/* Now we need to look at the real coupon dates */
-
-	prev = xl ? coup_cd_xl (settlement, maturity, freq, FALSE) :
-		coup_cd (settlement, maturity, freq, oem, FALSE);
-	next = xl ? coup_cd_xl (settlement, maturity, freq, TRUE) :
-		coup_cd (settlement, maturity, freq, oem, TRUE);
-	adjust_dates_basis (prev, next, basis);
-	days = days_between_dep_basis (prev, next, basis);
-	g_date_free (prev);
-	g_date_free (next);
-	return days;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -566,18 +527,15 @@ coupdays (GDate *settlement, GDate *maturity, int freq, int basis, gboolean oem,
  * Returns the number of days from the beginning of the coupon period to
  * the settlement date.
  */
-int
-coupdaybs (GDate *settlement, GDate *maturity, int freq, int basis, gboolean oem, 
-	   gboolean xl)
+gnum_float
+coupdaybs (GDate *settlement, GDate *maturity, int freq, basis_t basis, gboolean eom)
 {
 	GDate      *prev_coupon;
-	int        days;
+	gint32        days;
 
-	prev_coupon = xl ? coup_cd_xl (settlement, maturity, freq, FALSE) :
-		coup_cd (settlement, maturity, freq, oem, FALSE);
-	adjust_dates_basis (settlement, prev_coupon, basis);
-	days = - days_between_dep_basis (settlement, prev_coupon, basis);
-	g_date_free (prev_coupon);
+	prev_coupon = coup_cd (settlement, maturity, freq, eom, FALSE);
+	days = days_between_basis (prev_coupon, settlement, basis);
+	datetime_g_free (prev_coupon);
 	return days;
 }
 
@@ -589,18 +547,15 @@ coupdaybs (GDate *settlement, GDate *maturity, int freq, int basis, gboolean oem
  * coupon date.
  */
 
-int
-coupdaysnc (GDate *settlement, GDate *maturity, int freq, int basis, gboolean oem, 
-	    gboolean xl)
+gnum_float
+coupdaysnc (GDate *settlement, GDate *maturity, int freq, basis_t basis, gboolean eom)
 {
 	GDate      *next_coupon;
 	int        days;
 
-	next_coupon = xl ? coup_cd_xl (settlement, maturity, freq, TRUE) :
-		coup_cd (settlement, maturity, freq, oem, TRUE);
-	adjust_dates_basis (settlement, next_coupon, basis);
-	days = days_between_dep_basis (settlement, next_coupon, basis);
-	g_date_free (next_coupon);
+	next_coupon = coup_cd (settlement, maturity, freq, eom, TRUE);
+	days = days_between_basis (settlement, next_coupon, basis);
+	datetime_g_free (next_coupon);
 	return days;
 }
 
