@@ -288,8 +288,8 @@ expr_name_create (ParsePos const *pp, char const *name,
 	NamedExpression *res;
 	char const *err = NULL;
 	ExprTree *tree;
-	
-	tree = expr_parse_str (value, pp, GNM_PARSER_DEFAULT, NULL, error);
+
+	tree = expr_parse_str (value, pp, GNM_PARSER_DEFAULT, error);
 	if (!tree)
 		return NULL;
 
@@ -371,7 +371,7 @@ expr_name_remove (NamedExpression *nexpr)
 	g_return_if_fail (nexpr != NULL);
 	g_return_if_fail (nexpr->active);
 
-	/* Ref it so that we can clear it even if it is unused */
+	/* ref so that we can safely clear the expression later */
 	expr_name_ref (nexpr);
 	expr_name_unlink (nexpr);
 	expr_name_set_expr (nexpr, NULL);
@@ -434,9 +434,9 @@ expr_name_eval (NamedExpression const *nexpr,
 	if (nexpr->builtin) {
 		FunctionEvalInfo ei;
 		ei.pos = pos;
-		ei.func_def = NULL; /* FIXME : This is ugly. why are there
+		ei.func_call = NULL; /* FIXME : This is ugly. why are there
 					no descriptors for builtins */
-		return nexpr->t.expr_func (&ei, NULL);
+		return (*nexpr->t.expr_func) (&ei, NULL);
 	}
 
 	return expr_eval (nexpr->t.expr_tree, pos, flags);
@@ -447,9 +447,9 @@ expr_name_eval (NamedExpression const *nexpr,
  */
 /**
  * expr_name_set_scope:
- * @nexpr: 
- * @sheet: 
- * 
+ * @nexpr:
+ * @sheet:
+ *
  * Return Value: FALSE or error, TRUE otherwise
  **/
 gboolean
@@ -458,10 +458,10 @@ expr_name_set_scope (NamedExpression *nexpr, Sheet *sheet)
 	Workbook *wb;
 
 	g_return_val_if_fail (IS_SHEET (nexpr->pos.sheet), FALSE);
-	
+
 	sheet = nexpr->pos.sheet;
 	wb = sheet->workbook;
-	
+
 	nexpr->pos.sheet = NULL;
 	nexpr->pos.wb = wb;
 
@@ -472,7 +472,7 @@ expr_name_set_scope (NamedExpression *nexpr, Sheet *sheet)
 
 	return TRUE;
 }
-	
+
 void
 expr_name_set_expr (NamedExpression *nexpr, ExprTree *new_expr)
 {
@@ -490,6 +490,7 @@ expr_name_set_expr (NamedExpression *nexpr, ExprTree *new_expr)
 	}
 	nexpr->t.expr_tree = new_expr;
 	expr_name_link_deps (deps);
+
 	if (new_expr != NULL)
 		expr_name_handle_references (nexpr, TRUE);
 }
@@ -531,9 +532,9 @@ name_sheet_title (FunctionEvalInfo *ei, Value **args)
 }
 
 static struct {
-	gchar        *name;
-	FunctionArgs *fn;
-} builtins[] =
+	gchar const *name;
+	FunctionArgs fn;
+} const builtins[] =
 {
 	/* Consolidate_Area
 	   Auto_Open
