@@ -1,3 +1,5 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+
 /*
  * func.c: Function management and utility routines.
  *
@@ -18,6 +20,8 @@
 #include "symbol.h"
 #include "workbook.h"
 #include "sheet.h"
+#include "value.h"
+#include "expr.h"
 #include "number-match.h"
 #include "format.h"
 
@@ -27,41 +31,6 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-/* These are not supported yet */
-typedef enum {
-	FUNCTION_RETURNS_ARRAY = 0x01, /* eg transpose(), mmult() */
-	FUNCTION_RECALC_ALWAYS = 0x02, /* eg now() */
-
-	/* For functions that are not exactly compatible with various import
-	 * formats.  We need to recalc their results to avoid changing values
-	 * unexpectedly when we recalc later.  This probably needs to be done
-	 * on a per import format basis.  It may not belong here.
-	 */
-	FUNCTION_RECALC_ONLOAD = 0x04
-
-	/* TODO : Are there other forms or recalc we need to think about ? */
-} FunctionFlags;
-
-struct _FunctionDefinition {
-	FunctionGetFullInfoCallback get_full_info_callback;
-	FunctionFlags flags;
-	gchar   const *name;
-	gchar   const *named_arguments;
-	gchar        **help;
-	FuncType       fn_type;
-	union {
-		FunctionNodes *fn_nodes;
-		struct {
-			const char *arg_spec;
-			FunctionArgs  *func;
-			int min_args, max_args;
-			char *arg_types;
-		} args;
-	} fn;
-	gpointer     user_data;
-	gint         ref_count;
-};
-
 static GList *categories = NULL;
 static SymbolTable *global_symbol_table = NULL;
 
@@ -100,9 +69,12 @@ copy_hash_table_to_ptr_array (gpointer key, gpointer value, gpointer array)
 {
 	Symbol *sym = value;
 	FunctionDefinition *fd = sym->data;
-	if (sym->type == SYMBOL_FUNCTION &&
-	    fd->help != NULL && fd->name != NULL)
-		g_ptr_array_add (array, fd);
+	if (sym->type == SYMBOL_FUNCTION && fd->name != NULL) {
+		if (fd->fn_type == FUNCTION_NAMEONLY)
+			func_def_load ((FunctionDefinition *) fd);
+		if (fd->help != NULL)
+			g_ptr_array_add (array, fd);
+	}
 }
 
 static int
@@ -115,7 +87,7 @@ func_def_cmp (gconstpointer a, gconstpointer b)
 }
 
 void
-function_dump_defs (const char *filename)
+function_dump_defs (char const *filename)
 {
 	FILE *output_file;
 	unsigned i;
@@ -284,37 +256,46 @@ error_function_no_full_info (FunctionEvalInfo *ei, ExprList *expr_node_list)
 	return value_new_error (ei->pos, _("Function implementation not available."));
 }
 
-static void
-function_def_get_full_info_if_needed (FunctionDefinition *fn_def)
+void
+func_def_load (FunctionDefinition *fn_def)
 {
-	if (fn_def->fn_type == FUNCTION_NAMEONLY) {
-		gchar *args, *arg_names, **help;
-		FunctionArgs *fn_args;
-		FunctionNodes *fn_nodes;
-		gboolean success;
+	gchar const *args;
+	gchar const *arg_names;
+	gchar const **help;
+	FunctionArgs	 fn_args;
+	FunctionNodes	 fn_nodes;
+	FuncLinkHandle	 fn_link;
+	FuncUnlinkHandle fn_unlink;
+	gboolean success;
 
-		success = fn_def->get_full_info_callback (
-		          fn_def, &args, &arg_names, &help,
-		          &fn_args, &fn_nodes);
-		if (success) {
-			fn_def->named_arguments = arg_names;
-			fn_def->help = help;
-			if (fn_args != NULL) {
-				fn_def->fn_type = FUNCTION_ARGS;
-				fn_def->fn.args.func = fn_args;
-				fn_def->fn.args.arg_spec = args;
-				extract_arg_types (fn_def);
-			} else if (fn_nodes != NULL) {
-				fn_def->fn_type = FUNCTION_NODES;
-				fn_def->fn.fn_nodes = fn_nodes;
-			} else {
-				g_assert_not_reached ();
-			}
-		} else {
-			fn_def->named_arguments = "";
+	g_return_if_fail (fn_def->fn_type == FUNCTION_NAMEONLY);
+
+	success = fn_def->get_full_info_callback (
+		  fn_def, &args, &arg_names, &help,
+		  &fn_args, &fn_nodes, &fn_link, &fn_unlink);
+
+	if (success) {
+		fn_def->named_arguments = arg_names;
+		fn_def->help = help;
+		if (fn_args != NULL) {
+			fn_def->fn_type = FUNCTION_ARGS;
+			fn_def->fn.args.func = fn_args;
+			fn_def->fn.args.arg_spec = args;
+			extract_arg_types (fn_def);
+		} else if (fn_nodes != NULL) {
 			fn_def->fn_type = FUNCTION_NODES;
-			fn_def->fn.fn_nodes = &error_function_no_full_info;
+			fn_def->fn.fn_nodes = fn_nodes;
+		} else {
+			g_assert_not_reached ();
 		}
+		fn_def->link = fn_link;
+		fn_def->unlink = fn_unlink;
+	} else {
+		fn_def->named_arguments = "";
+		fn_def->fn_type = FUNCTION_NODES;
+		fn_def->fn.fn_nodes = &error_function_no_full_info;
+		fn_def->link = NULL;
+		fn_def->unlink = NULL;
 	}
 }
 
@@ -385,16 +366,18 @@ static FunctionDefinition *
 fn_def_new (FunctionCategory *category,
 	    char const *name,
 	    char const *arg_names,
-	    char **help)
+	    char const **help)
 {
 	FunctionDefinition *fn_def;
 
 	fn_def = g_new (FunctionDefinition, 1);
 	fn_def->get_full_info_callback = NULL;
-	fn_def->flags	 = 0;
-	fn_def->name      = name;
-	fn_def->help      = help;
+	fn_def->flags	= 0;
+	fn_def->name    = name;
+	fn_def->help    = help;
 	fn_def->named_arguments = arg_names;
+	fn_def->link	= NULL;
+	fn_def->unlink	= NULL;
 	fn_def->user_data = NULL;
 	fn_def->ref_count = 0;
 
@@ -410,8 +393,8 @@ function_add_args (FunctionCategory *category,
 		   char const *name,
 		   char const *args,
 		   char const *arg_names,
-		   char **help,
-		   FunctionArgs *fn)
+		   char const **help,
+		   FunctionArgs fn)
 {
 	static char const valid_tokens[] = "fsbraAS?|";
 	FunctionDefinition *fn_def;
@@ -440,8 +423,8 @@ function_add_nodes (FunctionCategory *category,
 		    char const *name,
 		    char const *args,
 		    char const *arg_names,
-		    char **help,
-		    FunctionNodes *fn)
+		    char const **help,
+		    FunctionNodes fn)
 {
 	FunctionDefinition *fn_def;
 
@@ -516,7 +499,7 @@ function_add_placeholder (char const *name, char const *type)
 }
 
 gpointer
-function_def_get_user_data (const FunctionDefinition *fn_def)
+function_def_get_user_data (FunctionDefinition const *fn_def)
 {
 	g_return_val_if_fail (fn_def != NULL, NULL);
 
@@ -532,7 +515,7 @@ function_def_set_user_data (FunctionDefinition *fn_def,
 	fn_def->user_data = user_data;
 }
 
-const char *
+char const *
 function_def_get_name (FunctionDefinition const *fn_def)
 {
 	g_return_val_if_fail (fn_def != NULL, NULL);
@@ -557,7 +540,7 @@ void
 function_def_count_args (FunctionDefinition const *fn_def,
                          int *min, int *max)
 {
-	const char *ptr;
+	char const *ptr;
 	int   i;
 	int   vararg;
 
@@ -565,7 +548,8 @@ function_def_count_args (FunctionDefinition const *fn_def,
 	g_return_if_fail (max != NULL);
 	g_return_if_fail (fn_def != NULL);
 
-	function_def_get_full_info_if_needed ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == FUNCTION_NAMEONLY)
+		func_def_load ((FunctionDefinition *) fn_def);
 
 	/*
 	 * FIXME: clearly for 'nodes' functions many of
@@ -590,6 +574,26 @@ function_def_count_args (FunctionDefinition const *fn_def,
 		*min = i;
 }
 
+/**
+ * function_set_link_handlers :
+ *
+ * Add callbacks for a function to perform special handling as each instance
+ * of the function is linked or unlinked from the sheet.
+ */
+void
+function_set_link_handlers (FunctionDefinition *fn_def,
+			    FuncLinkHandle   link,
+			    FuncUnlinkHandle unlink)
+{
+	/* Be paranoid for now */
+	g_return_if_fail (fn_def != NULL);
+	g_return_if_fail (fn_def->link == NULL);
+	g_return_if_fail (fn_def->unlink == NULL);
+
+	fn_def->link = link;
+	fn_def->unlink = unlink;
+}
+
 /**
  * function_def_get_arg_type:
  * @fn_def: the fn defintion
@@ -601,12 +605,13 @@ char
 function_def_get_arg_type (FunctionDefinition const *fn_def,
                            int arg_idx)
 {
-	const char *ptr;
+	char const *ptr;
 
 	g_return_val_if_fail (arg_idx >= 0, '?');
 	g_return_val_if_fail (fn_def != NULL, '?');
 
-	function_def_get_full_info_if_needed ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == FUNCTION_NAMEONLY)
+		func_def_load ((FunctionDefinition *) fn_def);
 
 	for (ptr = fn_def->fn.args.arg_spec; ptr && *ptr; ptr++) {
 		if (*ptr == '|')
@@ -787,12 +792,13 @@ function_call_with_list (FunctionEvalInfo *ei, ExprList *l)
 	Value **values;
 
 	g_return_val_if_fail (ei != NULL, NULL);
-	g_return_val_if_fail (ei->func_def != NULL, NULL);
+	g_return_val_if_fail (ei->func_call != NULL, NULL);
 
-	function_def_get_full_info_if_needed ((FunctionDefinition *) ei->func_def);
+	fn_def = ei->func_call->func;
+	if (fn_def->fn_type == FUNCTION_NAMEONLY)
+		func_def_load ((FunctionDefinition *) fn_def);
 
 	/* Functions that deal with ExprNodes */
-	fn_def = ei->func_def;
 	if (fn_def->fn_type == FUNCTION_NODES)
 		return fn_def->fn.fn_nodes (ei, l);
 
@@ -836,7 +842,7 @@ function_call_with_list (FunctionEvalInfo *ei, ExprList *l)
  * you have to compute/expand all of the values to use this
  */
 Value *
-function_call_with_values (const EvalPos *ep, const char *fn_name,
+function_call_with_values (EvalPos const *ep, char const *fn_name,
 			   int argc, Value *values [])
 {
 	FunctionDefinition *fn_def;
@@ -855,16 +861,19 @@ function_call_with_values (const EvalPos *ep, const char *fn_name,
 Value *
 function_def_call_with_values (EvalPos const *ep,
                                FunctionDefinition const *fn_def,
-                               gint                argc,
-                               Value              *values [])
+                               gint    argc,
+                               Value  *values [])
 {
 	Value *retval;
+	ExprFunction	ef;
 	FunctionEvalInfo fs;
 
-	function_def_get_full_info_if_needed ((FunctionDefinition *) fn_def);
-
 	fs.pos = ep;
-	fs.func_def = fn_def;
+	fs.func_call = &ef;
+	ef.func = (FunctionDefinition *)fn_def;
+
+	if (fn_def->fn_type == FUNCTION_NAMEONLY)
+		func_def_load (ef.func);
 
 	if (fn_def->fn_type == FUNCTION_NODES) {
 		/*
@@ -925,6 +934,7 @@ cb_iterate_cellrange (Sheet *sheet, int col, int row,
 
 	if (cell == NULL) {
 		ep.sheet = sheet;
+		ep.dep = NULL;
 		ep.eval.col = col;
 		ep.eval.row = row;
 		return (*data->callback)(&ep, NULL, data->closure);
@@ -947,12 +957,12 @@ cb_iterate_cellrange (Sheet *sheet, int col, int row,
  * Helper routine for function_iterate_argument_values.
  */
 Value *
-function_iterate_do_value (EvalPos      const *ep,
+function_iterate_do_value (EvalPos const *ep,
 			   FunctionIterateCB  callback,
-			   void                    *closure,
-			   Value                   *value,
-			   gboolean                 strict,
-			   gboolean		   ignore_blank)
+			   void		*closure,
+			   Value	*value,
+			   gboolean      strict,
+			   gboolean	 ignore_blank)
 {
 	Value *res = NULL;
 
@@ -1040,7 +1050,7 @@ function_iterate_argument_values (EvalPos const		*ep,
 
 	for (; result == NULL && expr_node_list;
 	     expr_node_list = expr_node_list->next) {
-		ExprTree const * tree = (ExprTree const *) expr_node_list->data;
+		ExprTree const * tree = expr_node_list->data;
 		Value *val;
 
 		/* Permit empties and non scalars. We don't know what form the
@@ -1072,7 +1082,8 @@ tokenized_help_new (FunctionDefinition const *fn_def)
 
 	g_return_val_if_fail (fn_def != NULL, NULL);
 
-	function_def_get_full_info_if_needed ((FunctionDefinition *) fn_def);
+	if (fn_def->fn_type == FUNCTION_NAMEONLY)
+		func_def_load ((FunctionDefinition *) fn_def);
 
 	tok = g_new (TokenizedHelp, 1);
 	tok->fndef = fn_def;
@@ -1125,8 +1136,8 @@ tokenized_help_new (FunctionDefinition const *fn_def)
 /**
  * Use to find a token eg. "FUNCTION"'s value.
  **/
-const char *
-tokenized_help_find (TokenizedHelp *tok, const char *token)
+char const *
+tokenized_help_find (TokenizedHelp *tok, char const *token)
 {
 	int lp;
 
@@ -1134,7 +1145,7 @@ tokenized_help_find (TokenizedHelp *tok, const char *token)
 		return "Incorrect Function Description.";
 
 	for (lp = 0; lp + 1 < (int)tok->sections->len; lp++) {
-		const char *cmp = g_ptr_array_index (tok->sections, lp);
+		char const *cmp = g_ptr_array_index (tok->sections, lp);
 
 		if (g_strcasecmp (cmp, token) == 0){
 			return g_ptr_array_index (tok->sections, lp + 1);
