@@ -29,7 +29,6 @@
 #include "clipboard.h"
 
 #include <libgnome/gnome-i18n.h>
-#include <libgnome/gnome-mime.h>
 #ifdef ENABLE_BONOBO
 #  include "sheet-object-container.h"
 #endif
@@ -38,6 +37,19 @@
 
 static GnomeCanvasClass *gcanvas_parent_class;
 
+static gboolean
+gnm_canvas_guru_key (WorkbookControlGUI const *wbcg, GdkEventKey *event)
+{
+	GtkWidget *guru = wbcg_edit_has_guru (wbcg);
+	GtkWidget *entry = GTK_WIDGET (gnm_expr_entry_get_entry (wbcg_get_entry_logical (wbcg)));
+
+	if (guru == NULL)
+		return FALSE;
+
+	gtk_widget_event ((entry != NULL) ? entry : guru, (GdkEvent *) event);
+	return TRUE;
+}
+
 /*
  * key press event handler for the gnumeric sheet for the sheet mode
  */
@@ -72,62 +84,61 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 			? scg_rangesel_extend
 			: scg_rangesel_move;
 	} else {
+		if ((event->state & GDK_CONTROL_MASK) && (event->state & GDK_SHIFT_MASK)) {
+			char const *fmt = NULL;
+			char const *desc = NULL;
+
+			switch (event->keyval) {
+			case GDK_asciitilde :
+				fmt = cell_formats [FMT_NUMBER][0];
+				desc = _("Format as Number");
+				break;
+			case GDK_dollar :
+				fmt = cell_formats [FMT_CURRENCY][0];
+				desc = _("Format as Currency");
+				break;
+			case GDK_percent :
+				fmt = cell_formats [FMT_PERCENT][0];
+				desc = _("Format as Percentage");
+				break;
+			case GDK_asciicircum :
+				fmt = cell_formats [FMT_SCIENCE][0];
+				desc = _("Format as Scientific");
+				break;
+			case GDK_numbersign :
+				fmt = cell_formats [FMT_DATE][0];
+				desc = _("Format as Date");
+				break;
+			case GDK_at :
+				fmt = cell_formats [FMT_TIME][0];
+				desc = _("Format as Time");
+				break;
+			case GDK_exclam :
+				fmt = cell_formats [FMT_ACCOUNT][0];
+				desc = _("Format as alternative Number"); /* FIXME: Better descriptor */
+				break;
+
+			case GDK_ampersand :
+				workbook_cmd_mutate_borders (WORKBOOK_CONTROL (wbcg), sheet, TRUE);
+				return TRUE;
+			case GDK_underscore :
+				workbook_cmd_mutate_borders (WORKBOOK_CONTROL (wbcg), sheet, TRUE);
+				return TRUE;
+			}
+
+			if (fmt != NULL) {
+				MStyle *mstyle = mstyle_new ();
+
+				mstyle_set_format_text (mstyle, fmt);
+				cmd_format (WORKBOOK_CONTROL (wbcg), sheet, mstyle, NULL, desc);
+				return TRUE;
+			}
+		}
 		movefn = (event->state & GDK_SHIFT_MASK)
 			? scg_cursor_extend
 			: scg_cursor_move;
 	}
 
-	if ((event->state & GDK_CONTROL_MASK) && (event->state & GDK_SHIFT_MASK)) {
-		char const *fmt = NULL;
-		char const *desc = NULL;
-		
-		switch (event->keyval) {
-		case GDK_asciitilde :
-			fmt = cell_formats [FMT_NUMBER][0];
-			desc = _("Format as Number");
-			break;
-		case GDK_dollar :
-			fmt = cell_formats [FMT_CURRENCY][0];
-			desc = _("Format as Currency");
-			break;
-		case GDK_percent :
-			fmt = cell_formats [FMT_PERCENT][0];
-			desc = _("Format as Percentage");
-			break;
-		case GDK_asciicircum :
-			fmt = cell_formats [FMT_SCIENCE][0];
-			desc = _("Format as Scientific");
-			break;
-		case GDK_numbersign :
-			fmt = cell_formats [FMT_DATE][0];
-			desc = _("Format as Date");
-			break;
-		case GDK_at :
-			fmt = cell_formats [FMT_TIME][0];
-			desc = _("Format as Time");
-			break;
-		case GDK_exclam :
-			fmt = cell_formats [FMT_ACCOUNT][0];
-			desc = _("Format as alternative Number"); /* FIXME: Better descriptor */
-			break;
-
-		case GDK_ampersand :
-			workbook_cmd_mutate_borders (WORKBOOK_CONTROL (wbcg), sheet, TRUE);
-			return TRUE;
-		case GDK_underscore :
-			workbook_cmd_mutate_borders (WORKBOOK_CONTROL (wbcg), sheet, TRUE);
-			return TRUE;
-		}
-
-		if (fmt) {
-			MStyle *mstyle = mstyle_new ();
-
-			mstyle_set_format_text (mstyle, fmt);
-			cmd_format (WORKBOOK_CONTROL (wbcg), sheet, mstyle, NULL, desc);
-			return TRUE;
-		}
-	}
-	
 	switch (event->keyval) {
 	case GDK_KP_Left:
 	case GDK_Left:
@@ -211,7 +222,7 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 			scg_set_top_row (gcanvas->simple.scg, new_row);
 		} else {
 			Range r = sheet_get_extent (sheet, FALSE);
-		
+
 			/* do the ctrl-end jump to the extent in 2 steps */
 			(*movefn)(gcanvas->simple.scg, r.end.col - sheet->edit_pos.col, FALSE, TRUE);
 			if ((event->state & GDK_CONTROL_MASK))
@@ -221,6 +232,8 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 
 	case GDK_KP_Insert :
 	case GDK_Insert :
+		if (gnm_canvas_guru_key (wbcg, event))
+			break;
 		if (state == GDK_CONTROL_MASK)
 			sheet_selection_copy (WORKBOOK_CONTROL (wbcg), sheet);
 		else if (state == GDK_SHIFT_MASK)
@@ -229,6 +242,8 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 
 	case GDK_KP_Delete:
 	case GDK_Delete:
+		if (gnm_canvas_guru_key (wbcg, event))
+			break;
 		if (state == GDK_SHIFT_MASK) {
 			scg_mode_edit (sc);
 			sheet_selection_cut (WORKBOOK_CONTROL (wbcg), sheet);
@@ -248,18 +263,20 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 		     event->state == GDK_MOD1_MASK))
 			/* Forward the keystroke to the input line */
 			return gtk_widget_event (
-				GTK_WIDGET (wbcg_get_entry_logical (wbcg)),
+				GTK_WIDGET (gnm_expr_entry_get_entry (wbcg_get_entry_logical (wbcg))),
 				(GdkEvent *) event);
 		/* fall down */
 
 	case GDK_Tab:
 	case GDK_ISO_Left_Tab:
 	case GDK_KP_Tab:
-	{
+		if (gnm_canvas_guru_key (wbcg, event))
+			break;
+
 		/* Be careful to restore the editing sheet if we are editing */
 		if (wbcg->editing)
 			sheet = wbcg->editing_sheet;
-			
+
 		if (wbcg_edit_finish (wbcg, TRUE)) {
 			/* Figure out the direction */
 			gboolean const direction = (event->state & GDK_SHIFT_MASK) ? FALSE : TRUE;
@@ -269,21 +286,24 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 			sheet_selection_walk_step (sheet, direction, horizontal);
 		}
 		break;
-	}
 
 	case GDK_Escape:
 		wbcg_edit_finish (wbcg, FALSE);
 		application_clipboard_unant ();
 		break;
-		
+
 	case GDK_F4:
 		if (wbcg->editing)
-			return gtk_widget_event (GTK_WIDGET (wbcg_get_entry_logical (wbcg)),
+			return gtk_widget_event (GTK_WIDGET (gnm_expr_entry_get_entry (wbcg_get_entry_logical (wbcg))),
 						 (GdkEvent *) event);
 		return TRUE;
 
 	case GDK_F2:
-		wbcg_edit_start (wbcg, FALSE, FALSE);
+		if (gnm_canvas_guru_key (wbcg, event))
+			break;
+
+		if (!wbcg_edit_start (wbcg, FALSE, FALSE))
+			return FALSE; /* attempt to edit failed */
 		/* fall down */
 
 	case GDK_BackSpace:
@@ -303,12 +323,13 @@ gnm_canvas_key_mode_sheet (GnumericCanvas *gcanvas, GdkEventKey *event)
 			if (event->length == 0)
 				return FALSE;
 
-			wbcg_edit_start (wbcg, TRUE, TRUE);
+			if (!wbcg_edit_start (wbcg, TRUE, TRUE))
+				return FALSE; /* attempt to edit failed */
 		}
 		scg_rangesel_stop (gcanvas->simple.scg, FALSE);
 
 		/* Forward the keystroke to the input line */
-		return gtk_widget_event (GTK_WIDGET (wbcg_get_entry_logical (wbcg)),
+		return gtk_widget_event (GTK_WIDGET (gnm_expr_entry_get_entry (wbcg_get_entry_logical (wbcg))),
 					 (GdkEvent *) event);
 	}
 
@@ -324,11 +345,12 @@ static gboolean
 gnm_canvas_key_mode_object (GnumericCanvas *gcanvas, GdkEventKey *event)
 {
 	SheetControlGUI *scg = gcanvas->simple.scg;
+	SheetControl    *sc = SHEET_CONTROL (scg);
 	int size = (event->state & GDK_CONTROL_MASK) ? 1 : 10;
 
 	switch (event->keyval) {
 	case GDK_Escape:
-		scg_mode_edit ((SheetControl *) scg);
+		scg_mode_edit (sc);
 		application_clipboard_unant ();
 		return TRUE;;
 
@@ -340,22 +362,22 @@ gnm_canvas_key_mode_object (GnumericCanvas *gcanvas, GdkEventKey *event)
 			return TRUE;
 		}
 		if (scg->current_object != NULL) {
-			gtk_object_destroy (GTK_OBJECT (scg->current_object));
+			cmd_object_delete (sc->wbc, scg->current_object);
 			return TRUE;
 		}
 		break;
 
 	case GDK_KP_Left: case GDK_Left:
-		scg_object_nudge (gcanvas->simple.scg, -size, 0);
+		scg_object_nudge (scg, -size, 0);
 		return TRUE;
 	case GDK_KP_Right: case GDK_Right:
-		scg_object_nudge (gcanvas->simple.scg,  size, 0);
+		scg_object_nudge (scg,  size, 0);
 		return TRUE;
 	case GDK_KP_Up: case GDK_Up:
-		scg_object_nudge (gcanvas->simple.scg, 0, -size);
+		scg_object_nudge (scg, 0, -size);
 		return TRUE;
 	case GDK_KP_Down: case GDK_Down:
-		scg_object_nudge (gcanvas->simple.scg, 0,  size);
+		scg_object_nudge (scg, 0,  size);
 		return TRUE;
 
 	default:
@@ -420,9 +442,12 @@ gnm_canvas_key_release (GtkWidget *widget, GdkEventKey *event)
 static gint
 gnm_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 {
+#warning IM disabled
+#if 0
 	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (widget);
 	if (gcanvas->ic)
 		gdk_im_begin (gcanvas->ic, gcanvas->simple.canvas.layout.bin_window);
+#endif
 	return (*GTK_WIDGET_CLASS (gcanvas_parent_class)->focus_in_event)(widget, event);
 }
 
@@ -430,7 +455,10 @@ gnm_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 static gint
 gnm_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 {
+#warning IM disabled
+#if 0
 	gdk_im_end ();
+#endif
 	return (*GTK_WIDGET_CLASS (gcanvas_parent_class)->focus_out_event)(widget, event);
 }
 
@@ -485,7 +513,12 @@ gnm_canvas_filenames_dropped (GtkWidget        *widget,
 	SheetControl *sc = (SheetControl *) gcanvas->simple.scg;
 	WorkbookControl *wbc = sc->wbc;
 
+#warning IM disabled
+#if 0
 	names = gnome_uri_list_extract_filenames ((char *)selection_data->data);
+#else
+	names = NULL;
+#endif
 
 	for (tmp_list = names; tmp_list != NULL; tmp_list = tmp_list->next) {
 		gchar *file_name = tmp_list->data;
@@ -512,13 +545,15 @@ gnm_canvas_filenames_dropped (GtkWidget        *widget,
 static void
 gnm_canvas_realize (GtkWidget *widget)
 {
+	if (GTK_WIDGET_CLASS (gcanvas_parent_class)->realize)
+		(*GTK_WIDGET_CLASS (gcanvas_parent_class)->realize)(widget);
+
+#warning IM disabled
+#if 0
 	gint width, height;
 	GdkWindow *window;
 	GnumericCanvas *gcanvas;
 
-	if (GTK_WIDGET_CLASS (gcanvas_parent_class)->realize)
-		(*GTK_WIDGET_CLASS (gcanvas_parent_class)->realize)(widget);
-
 	window = widget->window;
 	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window, NULL, FALSE);
 
@@ -537,34 +572,29 @@ gnm_canvas_realize (GtkWidget *widget)
 			GDK_IM_STATUS_NOTHING;
 
 		if(widget->style && widget->style->font->type != GDK_FONT_FONTSET)
-		  supported_style &= ~GDK_IM_PREEDIT_POSITION;
+			supported_style &= ~GDK_IM_PREEDIT_POSITION;
 
 		attr->style = style = gdk_im_decide_style (supported_style);
 		attr->client_window = gcanvas->simple.canvas.layout.bin_window;
 
-		switch (style & GDK_IM_PREEDIT_MASK)
-		  {
-		  case GDK_IM_PREEDIT_POSITION:
-		    if (widget->style && widget->style->font->type != GDK_FONT_FONTSET)
-		      {
-		        g_warning ("over-the-spot style requires fontset");			        break;
-		      }
+		if ((style & GDK_IM_PREEDIT_MASK) == GDK_IM_PREEDIT_POSITION) {
+			if (widget->style && widget->style->font->type != GDK_FONT_FONTSET) {
+				g_warning ("over-the-spot style requires fontset");
+			} else {
+				gdk_window_get_size (attr->client_window, &width, &height);
+				height = widget->style->font->ascent +
+					widget->style->font->descent;
 
-		    gdk_window_get_size (attr->client_window, &width, &height);
-		    height = widget->style->font->ascent +
-		             widget->style->font->descent;
-
-		    attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
-		    attr->spot_location.x = 0;
-		    attr->spot_location.y = height;
-		    attr->preedit_area.x = 0;
-		    attr->preedit_area.y = 0;
-		    attr->preedit_area.width = width;
-		    attr->preedit_area.height = height;
-		    attr->preedit_fontset = widget->style->font;
-
-		    break;
-		  }
+				attrmask |= GDK_IC_PREEDIT_POSITION_REQ;
+				attr->spot_location.x = 0;
+				attr->spot_location.y = height;
+				attr->preedit_area.x = 0;
+				attr->preedit_area.y = 0;
+				attr->preedit_area.width = width;
+				attr->preedit_area.height = height;
+				attr->preedit_fontset = widget->style->font;
+			}
+		}
 
 		gcanvas->ic = gdk_ic_new (attr, attrmask);
 		if (gcanvas->ic != NULL) {
@@ -577,6 +607,7 @@ gnm_canvas_realize (GtkWidget *widget)
 		} else
 			g_warning ("Can't create input context.");
 	}
+#endif
 }
 
 static void
@@ -587,6 +618,8 @@ gnm_canvas_unrealize (GtkWidget *widget)
 	gcanvas = GNUMERIC_CANVAS (widget);
 	g_return_if_fail (gcanvas != NULL);
 
+#warning IM disabled
+#if 0
 	if (gcanvas->ic) {
 		gdk_ic_destroy (gcanvas->ic);
 		gcanvas->ic = NULL;
@@ -595,6 +628,7 @@ gnm_canvas_unrealize (GtkWidget *widget)
 		gdk_ic_attr_destroy (gcanvas->ic_attr);
 		gcanvas->ic_attr = NULL;
 	}
+#endif
 
 	(*GTK_WIDGET_CLASS (gcanvas_parent_class)->unrealize)(widget);
 }
@@ -638,8 +672,11 @@ gnm_canvas_init (GnumericCanvas *gcanvas)
 {
 	GnomeCanvas *canvas = GNOME_CANVAS (gcanvas);
 
+#warning IM disabled
+#if 0
 	gcanvas->ic = NULL;
 	gcanvas->ic_attr = NULL;
+#endif
 	gcanvas->first.col = gcanvas->last_full.col = gcanvas->last_visible.col = 0;
 	gcanvas->first.row = gcanvas->last_full.row = gcanvas->last_visible.row = 0;
 	gcanvas->first_offset.col = 0;
@@ -664,7 +701,7 @@ GnumericCanvas *
 gnumeric_canvas_new (SheetControlGUI *scg, GnumericPane *pane)
 {
 	static GtkTargetEntry const drag_types[] = {
-		{ "text/uri-list", 0, 0 },
+		{ (char *)"text/uri-list", 0, 0 },
 	};
 	static gint const n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
 
@@ -683,16 +720,16 @@ gnumeric_canvas_new (SheetControlGUI *scg, GnumericPane *pane)
 		GNUMERIC_CANVAS_FACTOR_X, GNUMERIC_CANVAS_FACTOR_Y);
 
 	/* Setup a test of Drag and Drop */
-	gtk_signal_connect (GTK_OBJECT (gcanvas),
+	g_signal_connect (G_OBJECT (gcanvas),
 		"drag_data_get",
-		GTK_SIGNAL_FUNC (gnm_canvas_drag_data_get), NULL);
+		G_CALLBACK (gnm_canvas_drag_data_get), NULL);
 	gtk_drag_dest_set (GTK_WIDGET (gcanvas),
 		GTK_DEST_DEFAULT_ALL,
 		drag_types, n_drag_types,
 		GDK_ACTION_COPY);
-	gtk_signal_connect (GTK_OBJECT (gcanvas),
+	g_signal_connect (G_OBJECT (gcanvas),
 		"drag_data_received",
-		GTK_SIGNAL_FUNC (gnm_canvas_filenames_dropped),
+		G_CALLBACK (gnm_canvas_filenames_dropped),
 		gcanvas);
 
 	root_group = GNOME_CANVAS_GROUP (GNOME_CANVAS (gcanvas)->root);
@@ -1030,7 +1067,7 @@ gcanvas_sliding_callback (gpointer data)
 			gcanvas->sliding_adjacent_h = FALSE;
 
 		if (slide_x) {
-			col = target_gcanvas->last_full.col + 
+			col = target_gcanvas->last_full.col +
 				col_scroll_step (gcanvas->sliding_dx);
 			if (col >= SHEET_MAX_COLS-1) {
 				col = SHEET_MAX_COLS-1;
@@ -1050,7 +1087,7 @@ gcanvas_sliding_callback (gpointer data)
 					col = gnm_canvas_find_col (gcanvas1, x, NULL);
 					slide_x = FALSE;
 				}
-			} 
+			}
 
 			if (col <= gcanvas1->first.col) {
 				col = gcanvas1->first.col;
@@ -1060,7 +1097,7 @@ gcanvas_sliding_callback (gpointer data)
 			col = 0;
 			slide_x = FALSE;
 		}
-	} 
+	}
 
 	if (gcanvas->sliding_dy > 0) {
 		GnumericCanvas *target_gcanvas = gcanvas;
@@ -1084,7 +1121,7 @@ gcanvas_sliding_callback (gpointer data)
 			gcanvas->sliding_adjacent_v = FALSE;
 
 		if (slide_y) {
-			row = target_gcanvas->last_full.row + 
+			row = target_gcanvas->last_full.row +
 				row_scroll_step (gcanvas->sliding_dy);
 			if (row >= SHEET_MAX_ROWS-1) {
 				row = SHEET_MAX_ROWS-1;
@@ -1104,7 +1141,7 @@ gcanvas_sliding_callback (gpointer data)
 					row = gnm_canvas_find_row (gcanvas3, y, NULL);
 					slide_y = FALSE;
 				}
-			} 
+			}
 
 			if (row <= gcanvas3->first.row) {
 				row = gcanvas3->first.row;
@@ -1145,7 +1182,7 @@ gcanvas_sliding_callback (gpointer data)
  * @gcanvas	 : The GnumericCanvas managing the scroll
  * @canvas	 : The Canvas the event comes from
  * @event	 : The motion event
- * @slide_flags	 : 
+ * @slide_flags	 :
  * @slide_handler: The handler when sliding
  * @user_data	 : closure data
  *
