@@ -3,12 +3,11 @@
 /*
  * sheet-style.c: storage mechanism for styles and eventually cells.
  *
- * Copyright (C) 2000, 2001 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2002 Jody Goldberg (jody@gnome.org)
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of version 2 of the GNU General Public License as published
+ * by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,22 +19,26 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
  */
-#include <config.h>
+#include <gnumeric-config.h>
+#include <string.h>
+#include "gnumeric.h"
 #include "sheet-style.h"
+
 #include "ranges.h"
 #include "sheet.h"
 #include "expr.h"
 #include "format.h"
 #include "style.h"
 #include "style-border.h"
+#include "style-color.h"
 #include "cell.h"
-#include "portability.h"
 
 typedef union _CellTile CellTile;
 struct _SheetStyleData {
 	GHashTable *style_hash;
 	CellTile   *styles;
 	MStyle	   *default_style;
+	StyleColor *auto_pattern_color;
 };
 
 /**
@@ -418,6 +421,10 @@ sheet_style_init (Sheet *sheet)
 	sheet->style_data = g_new (SheetStyleData, 1);
 	sheet->style_data->style_hash =
 		g_hash_table_new (mstyle_hash, (GCompareFunc) mstyle_equal);
+	sheet->style_data->auto_pattern_color = g_new (StyleColor, 1);
+	memcpy (sheet->style_data->auto_pattern_color,
+		style_color_auto_pattern (), sizeof (StyleColor));
+	sheet->style_data->auto_pattern_color->ref_count = 1;
 	sheet->style_data->default_style =
 		sheet_style_find (sheet, mstyle_new_default ());
 	sheet->style_data->styles =
@@ -454,12 +461,87 @@ sheet_style_shutdown (Sheet *sheet)
 	sheet->style_data->style_hash = NULL;
 	g_hash_table_foreach_remove (table, cb_unlink, NULL);
 	g_hash_table_destroy (table);
-
+	style_color_unref (sheet->style_data->auto_pattern_color);
 
 	g_free (sheet->style_data);
 	sheet->style_data = NULL;
 }
 
+/**
+ * sheet_style_set_auto_pattern_color
+ *
+ * @sheet:         The sheet
+ * @pattern_color: The color
+ *
+ * Set the color for rendering auto colored patterns in this sheet.
+ */
+void
+sheet_style_set_auto_pattern_color (Sheet  *sheet, StyleColor *pattern_color)
+{
+	StyleColor *apc;
+	int ref_count;
+
+	g_return_if_fail (IS_SHEET (sheet));
+	g_return_if_fail (sheet->style_data != NULL);
+
+	apc = sheet->style_data->auto_pattern_color;
+	ref_count = apc->ref_count;
+	memcpy(apc, pattern_color, sizeof (StyleColor));
+	apc->is_auto = TRUE;
+	apc->ref_count = ref_count;
+	style_color_unref (pattern_color);
+}
+
+/**
+ * sheet_style_get_auto_pattern_color:
+ *
+ * @sheet: the sheet
+ *
+ * Returns the color for rendering auto colored patterns in this sheet.
+ */
+StyleColor *
+sheet_style_get_auto_pattern_color (Sheet const *sheet)
+{
+	StyleColor *sc;
+	g_return_val_if_fail (IS_SHEET (sheet), style_color_black ());
+	g_return_val_if_fail (sheet->style_data != NULL, style_color_black ());
+	g_return_val_if_fail (sheet->style_data->auto_pattern_color != NULL,
+			      style_color_black ());
+
+	sc = sheet->style_data->auto_pattern_color;
+	style_color_ref (sc);
+
+	return sc;
+}
+
+/**
+ * This function updates the color of style_border_none when the sheet to be
+ * rendered is known. style_border_none tells how to render the
+ * grid. Because the grid color may be different for different sheets, the
+ * functions which render the grid call this function first.  The rule for
+ * selecting the grid color, which is the same as in Excel, is: - if the
+ * auto pattern color is default (which is black), the grid color is gray,
+ * as returned by style_color_grid ().  - otherwise, the auto pattern color
+ * is used for the grid.
+ */
+void
+sheet_style_update_grid_color (Sheet const *sheet)
+{
+	StyleColor *default_auto = style_color_auto_pattern ();
+	StyleColor *sheet_auto = sheet_style_get_auto_pattern_color (sheet);
+	StyleColor *new_color;
+
+	new_color = (style_color_equal (default_auto, sheet_auto)
+		     ? style_color_grid ()
+		     : sheet_auto);
+
+	/* Do nothing if we already have the right color */
+	if (style_border_none()->color != new_color)
+		style_border_none_set_color (new_color);
+	style_color_unref (sheet_auto);
+	style_color_unref (default_auto);
+}
+
 /****************************************************************************/
 
 static MStyle *
@@ -553,7 +635,7 @@ cell_tile_apply (CellTile **tile, int level,
 	g_return_if_fail (TILE_SIMPLE <= type && type <= TILE_PTR_MATRIX);
 
 	/* applying the same style to part of a simple-tile is a nop */
-	if (type == TILE_SIMPLE && 
+	if (type == TILE_SIMPLE &&
 	    (*tile)->style_simple.style [0] == rs->new_style)
 		return;
 
@@ -1034,7 +1116,7 @@ get_style_row (CellTile const *tile, int level,
 
 /**
  * sheet_style_get_row :
- * 
+ *
  * @sheet :
  * @sr    :
  *
@@ -1473,7 +1555,7 @@ sheet_style_relocate (ExprRelocateInfo const *rinfo)
 }
 
 /**
- * sheet_style_insert_colrow 
+ * sheet_style_insert_colrow
  *
  * @rinfo :
  *
@@ -1565,7 +1647,7 @@ cb_visible_content (MStyle *style,
  */
 gboolean
 sheet_style_has_visible_content (Sheet const *sheet, Range *src)
-{ 
+{
 	gboolean res = FALSE;
 	foreach_tile (sheet->style_data->styles,
 		      TILE_TOP_LEVEL, 0, 0, src,
@@ -1584,7 +1666,7 @@ sheet_style_has_visible_content (Sheet const *sheet, Range *src)
  */
 void
 sheet_style_get_extent (Sheet const *sheet, Range *res)
-{ 
+{
 	Range r;
 
 	/* This could easily be optimized */
@@ -1715,7 +1797,7 @@ cb_hash_to_list (gpointer key, gpointer	value, gpointer	user_data)
 		return TRUE;
 	}
 
-	*res = g_list_prepend (*res, value);
+	*res = g_slist_prepend (*res, value);
 	return FALSE;
 }
 
@@ -1751,7 +1833,7 @@ sheet_style_get_list (Sheet const *sheet, Range const *r)
 }
 
 /**
- * sheet_style_set_list 
+ * sheet_style_set_list
  *
  * @sheet     :
  * @corner    :
@@ -1763,10 +1845,10 @@ sheet_style_get_list (Sheet const *sheet, Range const *r)
  */
 SpanCalcFlags
 sheet_style_set_list (Sheet *sheet, CellPos const *corner,
-		      gboolean transpose, GList const *list)
+		      gboolean transpose, StyleList const *list)
 {
 	SpanCalcFlags spanflags = SPANCALC_SIMPLE;
-	GList const *l;
+	StyleList const *l;
 
 	g_return_val_if_fail (IS_SHEET (sheet), spanflags);
 
@@ -1801,7 +1883,7 @@ style_list_free (StyleList *list)
 
 	for (l = list; l; l = l->next)
 		style_region_free (l->data);
-	g_list_free (list);
+	g_slist_free (list);
 }
 
 /**
