@@ -21,18 +21,20 @@
 
 #include <gnumeric-config.h>
 #include "gnumeric.h"
+#include "numbers.h"
+#include "mathfunc.h"
 #include "validation.h"
 #include "expr.h"
 #include "mstyle.h"
 #include "sheet.h"
 #include "cell.h"
 #include "value.h"
-#include "mathfunc.h"
 #include "number-match.h"
 #include "workbook-control.h"
 
 #include <libgnome/gnome-i18n.h>
 #include <math.h>
+#include <string.h>
 
 /**
  * validation_new :
@@ -54,7 +56,7 @@ validation_new (ValidationStyle style,
 
 	v = g_new0 (Validation, 1);
 	v->ref_count = 1;
-	
+
 	v->title = title ? string_get (title) : NULL;
 	v->msg   = msg ? string_get (msg) : NULL;
 	v->expr[0] = expr0;
@@ -83,7 +85,7 @@ validation_unref (Validation *v)
 	g_return_if_fail (v != NULL);
 
 	v->ref_count--;
-	
+
 	if (v->ref_count < 1) {
 		if (v->title != NULL) {
 			string_unref (v->title);
@@ -153,11 +155,11 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 			/* we know it is not empty */
 			if (val->type == VALUE_ERROR) {
 				msg = g_strdup_printf (_("'%s' is an error"),
-						       val->v_err.mesg->str);
+						       value_peek_string (val));
 				break;
-			} else if (cell->value->type == VALUE_STRING) {
-				char const *s = value_peek_string (val);
-				res = format_match_number (s, NULL, NULL);
+			} else if (val->type == VALUE_STRING) {
+				const char *s = value_peek_string (val);
+				res = format_match_number (s, NULL);
 				if (res == NULL) {
 					char const *fmt;
 					/* FIXME what else is needed */
@@ -171,12 +173,12 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 					break;
 				}
 			} else
-				res = value_duplicate (cell->value);
+				res = value_duplicate (val);
 
 			if (v->type == VALIDATION_TYPE_AS_INT &&
 			    res != NULL && res->type == VALUE_FLOAT) {
 				gnum_float f = value_get_as_float (res);
-				gboolean isint = fabs (f - gnumeric_fake_round (f)) < 1e-10;
+				gboolean isint = gnumabs (f - gnumeric_fake_round (f)) < 1e-10;
 				if (!isint) {
 					char const *valstr = value_peek_string (val);
 					msg = g_strdup_printf (_("'%s' is not an integer"), valstr);
@@ -199,7 +201,7 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 			 * This seems wrong for
 			 */
 			val_expr = expr_tree_new_constant (
-				value_new_int (strlen (value_peek_string (cell->value))));
+				value_new_int (strlen (value_peek_string (val))));
 			break;
 
 		case VALIDATION_TYPE_CUSTOM :
@@ -224,7 +226,7 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 			case VALIDATION_OP_GTE :	 op = OPER_GTE;		break;
 			case VALIDATION_OP_LTE :	 op = OPER_LTE;		break;
 			default :
-				g_warning ("Invalid operator %d", v->op);
+				g_warning ("Invalid validation operator %d", v->op);
 				return VALIDATION_STATUS_VALID;
 			}
 
@@ -258,7 +260,7 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 				g_return_val_if_fail (v->expr[1] != NULL, VALIDATION_STATUS_VALID);
 
 				expr_tree_ref (v->expr[1]);
-				expr = expr_tree_new_binary (val_expr, 
+				expr = expr_tree_new_binary (val_expr,
 					(v->op == VALIDATION_OP_BETWEEN) ? OPER_LTE : OPER_GT,
 					v->expr[1]);
 				val = expr_eval (expr, &ep, EVAL_STRICT);
@@ -286,13 +288,13 @@ validation_eval (WorkbookControl *wbc, MStyle const *mstyle,
 			allocated_msg = TRUE;
 		else
 			msg = _("That value is invalid.\n"
-				"Restrictions have been placed on this cell's content.");
+				"Restrictions have been placed on this cell's contents.");
 	}
 
-	result = wb_control_validation_msg (wbc, v,
+	result = wb_control_validation_msg (wbc, v->type,
 		(v->title != NULL && v->title->str[0] != '\0')
 			? v->title->str
-			: _("Gnumeric : Validation"),
+			: _("Gnumeric: Validation"),
 		msg);
 	if (allocated_msg)
 		g_free (msg);
